"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const messages_1 = require("../util/messages");
const dielAstTypes_1 = require("../parser/dielAstTypes");
const distributeQueries_1 = require("./passes/distributeQueries");
const DerivedRelationTypes = new Set(["View" /* View */, "EventView" /* EventView */, , "Output" /* Output */, "DerivedTable" /* DerivedTable */]);
const OriginalRelationTypes = new Set(["Table" /* Table */, "EventTable" /* EventTable */, "ExistingAndImmutable" /* ExistingAndImmutable */]);
// --------------- BEGIN CHECKERs --------------------
function IsRelationTypeDerived(rType) {
    if (DerivedRelationTypes.has(rType)) {
        return true;
    }
    else if (OriginalRelationTypes.has(rType)) {
        return false;
    }
    else {
        messages_1.LogInternalError(`RelationType ${rType} is not defined to be derived or not`);
        return null;
    }
}
exports.IsRelationTypeDerived = IsRelationTypeDerived;
// --------------- BEGIN GETTERS --------------------
function DeriveColumnsFromRelation(r) {
    if (IsRelationTypeDerived(r.relationType)) {
        const d = r;
        const originalColumns = DeriveColumnsFromSelectionUnit(d.selection.compositeSelections[0].relation);
        const derivedColumns = r.relationType === "EventView" /* EventView */
            // this casting is a bit brittle; should think more about the types
            ? originalColumns.concat(distributeQueries_1.EventViewColumns)
            : originalColumns;
        return derivedColumns;
    }
    else {
        const o = r;
        const originalColumns = o.columns.map(c => ({
            cName: c.cName,
            dataType: c.dataType
        }));
        const derivedColumns = r.relationType === "EventTable" /* EventTable */
            ? originalColumns.concat(distributeQueries_1.EventTableColumns)
            : originalColumns;
        return derivedColumns;
    }
}
exports.DeriveColumnsFromRelation = DeriveColumnsFromRelation;
function DeriveColumnsFromSelectionUnit(su) {
    const columns = [];
    if (!su.derivedColumnSelections)
        return messages_1.LogInternalError(`These should be defined already! Might not be visiting from topoligical order.`);
    for (let i = 0; i < su.derivedColumnSelections.length; i++) {
        const column = su.derivedColumnSelections[i];
        switch (column.expr.exprType) {
            case dielAstTypes_1.ExprType.Column:
                const columnExpr = column.expr;
                columns.push({
                    cName: column.alias,
                    dataType: columnExpr.dataType
                });
                break;
            case dielAstTypes_1.ExprType.Func:
                const functionExpr = column.expr;
                columns.push({
                    cName: column.alias,
                    dataType: functionExpr.dataType
                });
                break;
            default:
                return messages_1.LogInternalError(`${column.expr.exprType} Not handled`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
        }
    }
    return columns;
}
exports.DeriveColumnsFromSelectionUnit = DeriveColumnsFromSelectionUnit;
function GetAllDerivedViews(ast) {
    return ast.relations.filter(r => IsRelationTypeDerived(r.relationType));
}
exports.GetAllDerivedViews = GetAllDerivedViews;
function GetAllPrograms(ast) {
    return ast.programs;
}
exports.GetAllPrograms = GetAllPrograms;
function GetAllStaticOriginalTables(ast) {
    return ast.relations.filter(r => r.relationType === "ExistingAndImmutable" /* ExistingAndImmutable */);
}
exports.GetAllStaticOriginalTables = GetAllStaticOriginalTables;
function GetAllDielDefinedOriginalRelations(ast) {
    return ast.relations.filter(r => (r.relationType === "EventTable" /* EventTable */)
        || (r.relationType === "Table" /* Table */));
}
exports.GetAllDielDefinedOriginalRelations = GetAllDielDefinedOriginalRelations;
function GetOriginalRelations(ast) {
    return ast.relations.filter(r => !IsRelationTypeDerived(r.relationType));
}
exports.GetOriginalRelations = GetOriginalRelations;
function GetAllOutputs(ast) {
    return ast.relations.filter(r => r.relationType === "Output" /* Output */);
}
exports.GetAllOutputs = GetAllOutputs;
function GetRelationDef(ast, rName) {
    // first search in original, then serach in derived, compalin otherwise
    if (!rName)
        return messages_1.LogInternalError(`[GetRelationDef]: passed in null name`);
    const result = ast.relations.find(r => r.rName === rName);
    if (result) {
        return result;
    }
    else {
        return messages_1.LogInternalError(`[GetRelationDef]: Relation ${rName} not defined`);
    }
}
exports.GetRelationDef = GetRelationDef;
// --------------- BEGIN DERIVERS --------------------
// --------------  ---------------
function IsRelationEvent(ast, rName) {
    const r = GetRelationDef(ast, rName);
    return (r.relationType === "EventTable" /* EventTable */) || (r.relationType === "EventView" /* EventView */);
}
exports.IsRelationEvent = IsRelationEvent;
//# sourceMappingURL=DielAstGetters.js.map