"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dielAstTypes_1 = require("../parser/dielAstTypes");
const messages_1 = require("../util/messages");
const DielAstGetters_1 = require("./DielAstGetters");
const dependency_1 = require("./passes/dependency");
// --------------- BEGIN SETTERS --------------------
function AddRelation(ast, newR) {
    // make sure that the name is not found
    const idx = ast.relations.findIndex(r => r.rName === newR.rName);
    if (idx > -1) {
        messages_1.LogInternalError(`The relation you want to add, ${newR.rName}, is already defined`);
    }
    else {
        ast.relations.push(newR);
        // also need to update the dependency tree....
        if (DielAstGetters_1.IsRelationTypeDerived(newR.relationType)) {
            dependency_1.AddSingleDependencyByDerivedRelation(ast, newR);
        }
    }
}
exports.AddRelation = AddRelation;
function DeleteRelation(ast, relationName) {
    const idx = ast.relations.findIndex(r => r.rName === relationName);
    if (idx > -1) {
        return ast.relations.splice(idx, 1);
    }
    messages_1.LogInternalWarning(`Relation ${relationName} to delete was not found`);
    // also need to update the dependency tree....
    if (ast.depTree) {
        ast.depTree.delete(relationName);
    }
    return null;
}
exports.DeleteRelation = DeleteRelation;
// --------------- BEGIN VISITORS --------------------
// we are going to assume that the AST is sorted in topological order already
function WalkThroughSelectionUnits(ast, fun) {
    const results = [];
    function applyToDerivedRelation(r, fun) {
        r.selection.compositeSelections.map(c => {
            const result = fun(c.relation, ast, r.rName);
            results.push(result);
        });
    }
    const derived = DielAstGetters_1.GetAllDerivedViews(ast);
    derived.map(r => {
        applyToDerivedRelation(r, fun);
    });
    ast.programs.forEach((commands, _) => {
        commands.map(c => {
            // check the select clause from insert
            if (c.astType === dielAstTypes_1.AstType.Insert) {
                const insertClause = c;
                if (insertClause.selection) {
                    insertClause.selection.compositeSelections.map(s => fun(s.relation, ast, undefined));
                }
            }
        });
    });
    return results;
}
exports.WalkThroughSelectionUnits = WalkThroughSelectionUnits;
// FIXME: I think there are other places with Relation Reference.. should fix later
function WalkThroughRelationReferences(ast, fun) {
    const results = [];
    const selectionUnitFunc = (s) => {
        if (s.baseRelation) {
            results.push(fun(s.baseRelation));
            if (s.joinClauses) {
                s.joinClauses.map(j => {
                    results.push(fun(j.relation));
                });
            }
        }
    };
    WalkThroughSelectionUnits(ast, selectionUnitFunc);
    return results;
}
exports.WalkThroughRelationReferences = WalkThroughRelationReferences;
//# sourceMappingURL=DielAstVisitors.js.map