"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const runtimeTypes_1 = require("../runtime/runtimeTypes");
const distributeQueries_1 = require("./passes/distributeQueries");
const messages_1 = require("../util/messages");
const dielUtils_1 = require("../util/dielUtils");
const sqlAstTypes_1 = require("../parser/sqlAstTypes");
const materialization_1 = require("./passes/materialization");
const asyncPolicies_1 = require("../runtime/asyncPolicies");
const dependency_1 = require("./passes/dependency");
const DielAstGetters_1 = require("./DielAstGetters");
const DielAstVisitors_1 = require("./DielAstVisitors");
exports.LocalDbId = 1;
// local helper function
// not using a set here because sets do not work well over complex objects...
function addRelationIfOnlyNotExist(relationDef, newRelation) {
    // TEMP
    if (!relationDef.find(r => r.rName === newRelation.rName)) {
        relationDef.push(newRelation);
        return true;
    }
    return false;
}
class DielPhysicalExecution {
    constructor(ast, metaData, getEventByTimestep, addRelationToDiel) {
        this.ast = ast;
        this.addRelationToDiel = addRelationToDiel;
        this.metaData = metaData;
        this.getEventByTimestep = getEventByTimestep;
        // get all the outputs and loop
        this.augmentedDep = new Map();
        this.cachedEventViews = new Set();
        this.augmentDepTree();
        // let's first figure out the shipping information
        this.distributions = [];
        this.distributedEval();
        // TMP
        console.log("current distributions", this.distributions);
        // organize it by db-engines so we know how to set it up
        // then construct the definitions
        this.sqlAstSpecPerDb = new Map();
        this.setAstSpecPerDbIfNotExist(exports.LocalDbId);
        this.getSqlAstSpecsFromDistribution(this.distributions);
    }
    RemoveDerivedAst(view) {
        // TODO: do some tests to make sure that this is robust...
        // if anything else depends on this.... hmm
        this.ast.depTree.delete(view.rName);
    }
    // right now just do a reset, but performance wise it's better to do incrementally
    // FIXME
    AddDerivedAst(view) {
        // this.augmentSingleDepTreeNode(view);
        this.augmentedDep = new Map();
        this.augmentDepTree();
    }
    setAstSpecPerDbIfNotExist(dbId) {
        const ast = this.sqlAstSpecPerDb.get(dbId);
        if (ast) {
            return ast;
        }
        else {
            let newAst = sqlAstTypes_1.CreateEmptySqlAst();
            this.sqlAstSpecPerDb.set(dbId, newAst);
            return newAst;
        }
    }
    // FIXME: need to look into whether this is giving things in the right order
    GetInstructionsToAddOutput(view) {
        // we need to first figure it out what relations we need to to define to what
        // and also change the triggers now
        // the dependency is already agumented
        const originalNode = this.ast.depTree.get(view.rName);
        if (!originalNode) {
            return null;
        }
        const augmentedNode = this.augmentDepTreeNode(originalNode, view.rName);
        if (augmentedNode)
            this.augmentedDep.set(view.rName, augmentedNode);
        // then figure out the additional distributions
        const newDistributions = this.distributedEvalForOutput(view);
        // then figure out the ASTs
        const overallNewRelations = [];
        newDistributions.map(distribution => {
            const newRelations = this.getExecutionSpecForSingleDistribution(distribution);
            if (newRelations) {
                const isAdded = this.addNewRelationsToExistingSpec(newRelations);
                isAdded.map((b, i) => {
                    if (b)
                        overallNewRelations.push(newRelations[i]);
                });
            }
        });
        return overallNewRelations;
    }
    // TODO
    GetInstructionsToRemoveQuery() {
        messages_1.LogInternalError("GetInstructionsToRemoveQuery TODO", messages_1.DielInternalErrorType.NotImplemented);
    }
    /**
     * Distributed execution core
     * the logic is as follows:
     * if the from is a different location from the to,
     *   then make the destination a table, and the original location
     * else
     *   if the relation is static and immutable, do nothing
     *      else, define it
     * YIFAN TODO: this needs testing!
     */
    getExecutionSpecForSingleDistribution(distribution) {
        const rDef = DielAstGetters_1.GetRelationDef(this.ast, distribution.relationName);
        // const isLocal = distribution.from === distribution.to;
        if ((!distribution.to) || (!distribution.from)) {
            return messages_1.LogInternalError(`to and from should be defined by now!`);
        }
        if (!rDef)
            return messages_1.LogInternalError(`${distribution.relationName} Not found`);
        switch (rDef.relationType) {
            case "DerivedTable" /* DerivedTable */:
                // same for both cases
                const relationDef = distributeQueries_1.GetSqlOriginalRelationFromDielRelation(rDef);
                if (relationDef)
                    return [{ dbId: distribution.to, relationDef }];
                return null;
            // the following two cases should be the same
            case "Table" /* Table */:
            case "EventTable" /* EventTable */: {
                const addTimeColumns = (rDef.relationType === "EventTable" /* EventTable */) && (distribution.to === exports.LocalDbId);
                const eventTableDef = distributeQueries_1.GetSqlOriginalRelationFromDielRelation(rDef, addTimeColumns);
                if (eventTableDef)
                    return [
                        { dbId: distribution.from, relationDef: eventTableDef },
                        { dbId: distribution.to, relationDef: eventTableDef }
                    ];
            }
            // the following two cases should be the same if caching is disabled
            case "EventView" /* EventView */: {
                if (this.metaData.cache === true
                    && distribution.from !== distribution.to
                    && distribution.to === exports.LocalDbId
                    && isEventViewCacheable(rDef, this.metaData.relationLocation)) {
                    console.log(`${rDef.rName} is cacheable`);
                    this.cachedEventViews.add(rDef.rName);
                    const addTimeColumns = (rDef.relationType === "EventView" /* EventView */)
                        && (distribution.to === exports.LocalDbId);
                    const cacheRelationTriplet = distributeQueries_1.GetCachedEventView(rDef, addTimeColumns);
                    const eventView = distributeQueries_1.GetSqlDerivedRelationFromDielRelation(rDef);
                    if (eventView && cacheRelationTriplet) {
                        // TODO: need to add the cache relations to the AST!
                        // also the AST now need to be aware of the new input events...
                        this.addRelationToDiel(cacheRelationTriplet.cacheDielTable);
                        this.addRelationToDiel(cacheRelationTriplet.referenceDielTable);
                        this.addRelationToDiel(cacheRelationTriplet.dielView);
                        return [
                            { dbId: distribution.to, relationDef: cacheRelationTriplet.cacheTable },
                            { dbId: distribution.to, relationDef: cacheRelationTriplet.referenceTable },
                            { dbId: distribution.to, relationDef: cacheRelationTriplet.view },
                            { dbId: distribution.from, relationDef: eventView },
                        ];
                    }
                }
            } // else, fallthrough
            case "View" /* View */: {
                const addTimeColumns = (rDef.relationType === "EventView" /* EventView */) && (distribution.to === exports.LocalDbId);
                const eventViewTable = distributeQueries_1.GetSqlOriginalRelationFromDielRelation(rDef, addTimeColumns);
                const eventView = distributeQueries_1.GetSqlDerivedRelationFromDielRelation(rDef);
                if (eventView && eventViewTable)
                    return [
                        { dbId: distribution.from, relationDef: eventView },
                        { dbId: distribution.to, relationDef: eventViewTable }
                    ];
            }
            case "ExistingAndImmutable" /* ExistingAndImmutable */: {
                if (distribution.to !== distribution.from) {
                    const newTable = distributeQueries_1.GetSqlOriginalRelationFromDielRelation(rDef);
                    if (newTable)
                        return [{ dbId: distribution.to, relationDef: newTable }];
                }
                return null;
            }
            case "Output" /* Output */: {
                // both from and to must be local, and if they are not, we need to transform to create default policy
                // outputs can also be send to remotes (since they are also local views), but they cannot be sent in
                if ((distribution.from !== distribution.to) && (distribution.from !== exports.LocalDbId)) {
                    return messages_1.LogInternalError(`Outputs must be local`);
                }
                const relationDef = distributeQueries_1.GetSqlDerivedRelationFromDielRelation(rDef);
                if (relationDef) {
                    if (distribution.to !== exports.LocalDbId) {
                        const relationDefRemote = distributeQueries_1.GetSqlOriginalRelationFromDielRelation(rDef);
                        return [
                            { dbId: exports.LocalDbId, relationDef },
                            { dbId: distribution.to, relationDef: relationDefRemote }
                        ];
                    }
                    else {
                        return [{ dbId: exports.LocalDbId, relationDef }];
                    }
                }
            }
            default:
                return messages_1.LogInternalError(`Union types`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
        }
    }
    addNewRelationsToExistingSpec(newRelations) {
        return newRelations.map(newR => {
            const astSpec = this.setAstSpecPerDbIfNotExist(newR.dbId);
            return addRelationIfOnlyNotExist(astSpec.relations, newR.relationDef);
        });
    }
    getSqlAstSpecsFromDistribution(distributions) {
        const localAst = this.sqlAstSpecPerDb.get(exports.LocalDbId);
        if (!localAst) {
            messages_1.LogInternalError(`LocalAST must be defined`);
            return;
        }
        distributions.map(distribution => {
            const newRelations = this.getExecutionSpecForSingleDistribution(distribution);
            if (newRelations)
                this.addNewRelationsToExistingSpec(newRelations);
        });
        // need to add the static tables that are not directly referenced to main
        // find any static table that was not used by outputs...
        DielAstGetters_1.GetOriginalRelations(this.ast).map(r => {
            if ((r.relationType === "Table" /* Table */)
                || (r.relationType === "DerivedTable" /* DerivedTable */)) {
                if (!distributions.find(d => d.relationName === r.rName)) {
                    // we need to add this to the local one
                    // fixme: might be relevant for workers as well
                    const newOriginal = distributeQueries_1.GetSqlOriginalRelationFromDielRelation(r);
                    if (newOriginal) {
                        // need to do the following check because the distribution pass may also add relations
                        // sigh this is a bit messy.
                        if (!localAst.relations.find(r => r.rName === newOriginal.rName)) {
                            localAst.relations.push(newOriginal);
                        }
                    }
                }
            }
        });
        // as well as the commands
        localAst.commands = this.ast.commands;
        // add user defined programs to main db
        if (localAst.triggers.length > 0) {
            messages_1.LogInternalError("Triggers should not have been defined");
        }
        const triggers = [];
        const programsToAddRaw = this.ast.programs.get("");
        const programsToAdd = programsToAddRaw ? programsToAddRaw : [];
        this.ast.programs.forEach((v, input) => {
            triggers.push({
                tName: `${input}Trigger`,
                afterRelationName: input,
                commands: [...programsToAdd, ...v],
            });
        });
        localAst.triggers = triggers;
        // materialization pass
        this.sqlAstSpecPerDb.forEach(ast => {
            materialization_1.TransformAstForMaterialization(ast);
        });
    }
    /**
     * This is where we apply the default async policies
     * so if any of the relations are remote, we will have the relation
     * @param output
     */
    distributedEvalForOutput(output) {
        const newDistributions = [];
        const ast = this.ast;
        const scope = {
            augmentedDep: this.augmentedDep,
            selectRelationEvalOwner: this.selectRelationEvalOwner.bind(this),
            outputName: output.rName,
            relationTypeLookup: (rName) => DielAstGetters_1.GetRelationDef(ast, rName).relationType,
        };
        const result = distributeQueries_1.QueryDistributionRecursiveEval(newDistributions, scope, output.rName);
        if (!result)
            return null;
        if (result.fromDbId !== exports.LocalDbId) {
            // apply default policy!
            const eventDeps = dependency_1.DeriveOriginalRelationsAViewDependsOn(this.ast.depTree, output.rName);
            const v = asyncPolicies_1.OutputToAsyncDefaultPolicty(output, eventDeps);
            // also add the new definitions to the DIEL AST, modify in place?
            DielAstVisitors_1.DeleteRelation(this.ast, v.output.rName);
            DielAstVisitors_1.AddRelation(this.ast, v.asyncView);
            DielAstVisitors_1.AddRelation(this.ast, v.output);
            // if the from of the output is not local, we need to apply
            const outputDistribution = newDistributions.find(d => d.relationName === output.rName);
            if (!outputDistribution) {
                console.log(newDistributions);
                return messages_1.LogInternalError(`Should have shipping for output ${output.rName}`);
            }
            // need to modify existing distributions!
            // replace the output with the async view...
            const forOutput = newDistributions.filter(d => (d.forRelationName === output.rName) && (d.relationName !== output.rName));
            forOutput.map(f => f.forRelationName = v.asyncView.rName);
            newDistributions.push({
                forRelationName: v.asyncView.rName,
                relationName: v.asyncView.rName,
                from: result.fromDbId,
                to: exports.LocalDbId,
                finalOutputName: output.rName,
            });
            newDistributions.push({
                forRelationName: v.output.rName,
                relationName: v.asyncView.rName,
                from: exports.LocalDbId,
                to: exports.LocalDbId,
                finalOutputName: output.rName,
            });
            outputDistribution.from = exports.LocalDbId;
        }
        // newDistributions.push({
        //   forRelationName: output.rName,
        //   relationName: result.relationName,
        //   from: LocalDbId,
        //   to: LocalDbId,
        //   finalOutputName: output.rName,
        // });
        this.distributions = newDistributions.concat(this.distributions);
        return newDistributions;
    }
    /**
     * recursive function to evaluate what view needs to be where
     */
    distributedEval() {
        // let distributionsForAllOutput: SingleDistribution[] = [];
        DielAstGetters_1.GetAllOutputs(this.ast).map(this.distributedEvalForOutput.bind(this));
        // return distributionsForAllOutput;
    }
    /**
     * we want to figure out what relations to ship from the individual remotes
     * @param dbId
     * @param staticRelation
     * @param outputRelation
     */
    getBubbledUpRelationToShipForStatic(dbId, staticRelation, outputRelation) {
        const distributions = this.distributions;
        function helper(acc, dbId, relation) {
            distributions.map(d => {
                if ((d.relationName === relation) && (d.from === dbId)) {
                    if ((d.to === d.from) && (d.forRelationName !== relation)) {
                        helper(acc, d.to, d.forRelationName);
                    }
                    else if ((d.to !== d.from)) {
                        console.log("consider shipping", d);
                        // only push if
                        // - it's not already there
                        // - and it's for the output!
                        const hasAdded = acc.find(a => (a.destination === d.to) && (a.relation === d.relationName));
                        const isForOutput = d.finalOutputName === outputRelation;
                        if (!hasAdded && isForOutput) {
                            acc.push({
                                destination: d.to,
                                relation: d.relationName
                            });
                        }
                    }
                }
            });
        }
        let acc = [];
        helper(acc, dbId, staticRelation);
        return acc;
    }
    /**
     * this functions figures out what views are dependent on the new relation at the db location.
     * @param dbId
     * @param eventRelation
     */
    getBubbledUpRelationToShipForEvent(dbId, eventRelation, includedForRelation) {
        const distributions = this.distributions;
        let count = 0;
        function helper(acc, dbId, relation) {
            count += 1;
            if (count > 100) {
                debugger;
            }
            distributions.map(d => {
                // (d.finalOutputName === output)
                if ((d.relationName === relation) && (d.from === dbId)) {
                    if ((d.to === d.from) && (d.forRelationName !== relation)) {
                        helper(acc, d.to, d.forRelationName);
                    }
                    else if ((d.to !== d.from)) {
                        const isIncluded = !includedForRelation || (includedForRelation && (includedForRelation.find(r => r === d.forRelationName)));
                        const hasAdded = acc.find(a => (a.destination === d.to) && (a.relation === d.relationName));
                        if (!hasAdded && isIncluded) {
                            acc.push({
                                destination: d.to,
                                relation: d.relationName
                            });
                        }
                    }
                }
            });
        }
        let acc = [];
        helper(acc, dbId, eventRelation);
        return acc;
    }
    getStaticAsyncViewTrigger(outputName) {
        // also need to make sure that there are no input dependnecies at all
        const remoteSources = this.distributions
            .filter(d => ((d.finalOutputName === outputName)
            && (d.from !== exports.LocalDbId)
            && (d.to === d.from)));
        // const hasInput = remoteSources.reduce((acc, v) => acc || (v.from === LocalDbId), false);
        // if (hasInput) {
        // return null;
        // } else {
        const dedupeRelations = new Set();
        const result = [];
        remoteSources.map(d => {
            if (!dedupeRelations.has(d.relationName)) {
                dedupeRelations.add(d.relationName);
                result.push({
                    dbId: d.from,
                    relation: d.relationName
                });
            }
        });
        return result;
        // }
    }
    // TODO: we then need to fix to actually include event tables, as opposed to projecting them into tables
    // get the dependencies for the relationName, based on the input that has changed
    getRelationEventTableDependencies() {
    }
    /**
     * The goal of this function is to figure out for a specific engine, for a specific interaction what we need to ship.
     * Note that views might change too, if they are shipped, so the shipping logic is not limited the original tables
     * PERF FIXME: this can be materialized
     * @param dbId
     * @param requestTimestep
     */
    getRelationsToShipForDb(dbId, requestTimestep) {
        // a list of the views and their dependencies
        // if input is specified, we will filter dependencies by those relations that depend on the input
        const inputEvent = this.getEventByTimestep(requestTimestep);
        const allInputDeps = dependency_1.DeriveDependentRelations(this.ast.depTree, inputEvent);
        // the allInputDeps should also contain the original inputEvents as well.
        allInputDeps.add(inputEvent);
        if (!allInputDeps) {
            messages_1.LogInternalWarning(`There are no input dependencies for ${inputEvent}`);
            return null;
        }
        const ast = this.sqlAstSpecPerDb.get(dbId);
        if (!ast) {
            return messages_1.LogInternalError(`AST not defined for ${dbId}`);
        }
        const allEvents = ast.relations.filter(r => r.isDynamic).map(r => r.rName);
        // PERF FIXME: this is corase grained
        const deps = dielUtils_1.SetIntersection(new Set(allEvents), allInputDeps);
        const relationsToShip = new Map();
        this.distributions
            .filter(d => (d.from === dbId) && (d.from !== d.to))
            .map(d => {
            if (allInputDeps.has(d.relationName)) {
                const toShip = relationsToShip.get(d.relationName);
                if (!toShip) {
                    relationsToShip.set(d.relationName, new Set([d.to]));
                }
                else {
                    toShip.add(d.to);
                }
            }
        });
        return {
            deps,
            relationsToShip
        };
    }
    // this also needs to be recursive, because even if the inputs are not immediately shipped,
    // they might have dependencies down the line
    getShippingInfoForDbByEvent(eventTable, engineId) {
        const destinationDbIds = this.distributions
            .filter(d => ((d.relationName === eventTable)
            && (d.from === engineId)
            && (d.to !== d.from)))
            .map(d => d.to);
        // right now this is overly conservative, it would wait for all the things
        // might be empty if it does not need anything
        const relationsNeeded = this.distributions
            .filter(d => ((d.forRelationName === eventTable)
            && (d.to === engineId)
            && (d.to !== d.from)));
        return {
            destinationDbIds,
            relationsNeeded
        };
    }
    getLocalDbAst() {
        return this.getAstFromDbId(exports.LocalDbId);
    }
    getAstFromDbId(dbId) {
        return this.sqlAstSpecPerDb.get(dbId);
    }
    augmentDepTreeNode(nodDep, relationName) {
        const foundRelation = DielAstGetters_1.GetRelationDef(this.ast, relationName);
        if (!foundRelation)
            return messages_1.LogInternalError(`Relation ${relationName} is not found`);
        const relationType = foundRelation.relationType;
        let remoteId;
        if (relationType === "EventTable" /* EventTable */ || relationType === "Table" /* Table */) {
            remoteId = exports.LocalDbId;
        }
        else if (relationType === "ExistingAndImmutable" /* ExistingAndImmutable */) {
            const foundDbId = this.getDbIdByRelationName(relationName);
            if (!foundDbId) {
                return messages_1.LogInternalError(`Immutable table ${relationName} is not found`);
            }
            remoteId = foundDbId;
        }
        // undefined is fine, because we might not know, and need the recursive processing to set
        const nodeDepAugmetned = Object.assign({ remoteId,
            relationName }, Object.assign({}, nodDep));
        return nodeDepAugmetned;
    }
    augmentSingleDepTreeNode(nodeDep, relationName) {
        const nodeDepAugmetned = this.augmentDepTreeNode(nodeDep, relationName);
        if (nodeDepAugmetned) {
            this.augmentedDep.set(relationName, nodeDepAugmetned);
        }
        else {
            messages_1.LogInternalError(`Augmented tree is null for ${relationName}`);
        }
    }
    /**
     * adding where the location is, as well as the type of location it is.
     */
    augmentDepTree() {
        this.ast.depTree.forEach(this.augmentSingleDepTreeNode.bind(this));
    }
    // this function needs to access the metadata
    selectRelationEvalOwner(dbIds) {
        // right now just pick a socket if it's there
        // need to look up its information
        let workerId = null;
        for (let dbId of dbIds) {
            const dbMetaData = this.metaData.dbs.get(dbId);
            if (dbMetaData) {
                const dbType = dbMetaData.dbType;
                switch (dbType) {
                    case runtimeTypes_1.DbType.Socket: {
                        return dbId;
                    }
                    case runtimeTypes_1.DbType.Worker: {
                        workerId = dbId;
                    }
                }
            }
            else {
                debugger;
            }
        }
        if (workerId) {
            return workerId;
        }
        return exports.LocalDbId;
    }
    getRemoteInfoById(remoteId) {
        return this.sqlAstSpecPerDb.get(remoteId);
    }
    getDbIdByRelationName(rName) {
        // look up metadata
        const r = this.metaData.relationLocation.get(rName);
        if (r) {
            return r.dbId;
        }
        else {
            return messages_1.LogInternalError(`Cannot find relation ${rName}`);
        }
    }
}
exports.DielPhysicalExecution = DielPhysicalExecution;
// export and extracted for testing purposes
function dependsOnLocalTables(deps, relationLocations) {
    let dependsOnLocal = deps.some(d => {
        let loc = relationLocations.get(d);
        return loc === undefined;
    });
    return dependsOnLocal;
}
exports.dependsOnLocalTables = dependsOnLocalTables;
function dependsOnRemoteTables(deps, relationLocations) {
    let dependsOnRemote = deps.some(d => {
        let loc = relationLocations.get(d);
        return ((loc !== undefined) || (loc && loc.dbId !== exports.LocalDbId));
    });
    return dependsOnRemote;
}
exports.dependsOnRemoteTables = dependsOnRemoteTables;
// export and extracted for testing purposes
function dependsOnBothLocalAndForeignTables(deps, relationLocations) {
    return dependsOnLocalTables(deps, relationLocations) &&
        dependsOnRemoteTables(deps, relationLocations);
}
exports.dependsOnBothLocalAndForeignTables = dependsOnBothLocalAndForeignTables;
/**
 * Determines whether the event view can be cached.
 * This is determined by inpecting all of the join clauses and the
 * base relation. To be cacheable, all of these must either exclusively
 * reference local tables or exclusively reference foreign tables.
 * @param relation
 * @param relationLocations
 */
function isEventViewCacheable(relation, relationLocations) {
    if (relation.relationType !== "EventView" /* EventView */) {
        throw Error(`${relation.rName} is not an EventView!`);
        // TODO RYAN: Change error type
    }
    // need to check the "from" and the "joins"
    if (relation.selection.compositeSelections.length !== 1) {
        return false;
    } // totally arbitrary constraint; could change in future
    let mainSelection = relation.selection.compositeSelections[0].relation;
    if (dependsOnBothLocalAndForeignTables(dependency_1.getRelationReferenceDep(mainSelection.baseRelation), relationLocations)) {
        return false;
    }
    let joinRelations = mainSelection.joinClauses.map(j => j.relation);
    if (joinRelations.some(c => dependsOnBothLocalAndForeignTables(dependency_1.getRelationReferenceDep(c), relationLocations))) {
        return false;
    }
    else {
        return true;
    }
}
exports.isEventViewCacheable = isEventViewCacheable;
//# sourceMappingURL=DielPhysicalExecution.js.map