"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const messages_1 = require("../../util/messages");
const dielAstTypes_1 = require("../../parser/dielAstTypes");
// export function generateSqlFromDielAst(ast: DielAst, options?: { replace: boolean; isRemote: boolean}) {
//   const isRemote = options ? options.isRemote ? options.isRemote : false : false;
//   const sqlAst = createSqlAstFromDielAst(ast, isRemote);
//   const replace = options ? options.replace ? options.replace : false : false;
//   return generateStringFromSqlIr(sqlAst, replace);
// }
function generateCleanUpAstFromSqlAst(ast) {
    // basically drop everything
    // triggers etc.
    const tables = ast.relations.map(r => ({
        astType: dielAstTypes_1.AstType.Drop,
        dropType: r.relationType === "Table" /* Table */ ? dielAstTypes_1.DropType.Table : dielAstTypes_1.DropType.View,
        dropName: r.rName
    }));
    let triggers = ast.triggers.map(t => ({
        astType: dielAstTypes_1.AstType.Drop,
        dropType: dielAstTypes_1.DropType.Trigger,
        dropName: t.tName
    }));
    // note that we might need to do dependency order?
    return triggers.concat(tables);
}
exports.generateCleanUpAstFromSqlAst = generateCleanUpAstFromSqlAst;
function generateStringFromSqlIr(sqlAst, replace = false) {
    // if remoteType is server, then we need to drop the old ones if we want to make a new one
    // we need to architect this properly to scale, but a quick fix for now
    const relations = sqlAst.relations.map(t => GenerateSqlRelationString(t, replace));
    let triggers = sqlAst.triggers.map(t => generateTrigger({ trigger: t, replace }));
    let commands = sqlAst.commands.map(c => generateCommand(c));
    return relations.concat(triggers).concat(commands);
}
exports.generateStringFromSqlIr = generateStringFromSqlIr;
function generateCommand(command) {
    switch (command.astType) {
        case dielAstTypes_1.AstType.Delete:
            return generateDelete(command);
        case dielAstTypes_1.AstType.Insert:
            return generateInserts(command);
        case dielAstTypes_1.AstType.Drop:
            return generateDrop(command);
        case dielAstTypes_1.AstType.RelationSelection:
            return generateSelect(command.compositeSelections);
        default:
            messages_1.LogInternalError(`Other AstTypes ${command.astType} not handled`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
            return "";
    }
}
function generateDrop(command) {
    return `DROP ${command.dropType} ${command.dropName};`;
}
exports.generateDrop = generateDrop;
function generateDelete(command) {
    const pred = command.predicate
        ? `WHERE ${GetSqlStringFromExpr(command.predicate)}`
        : "";
    return `DELETE FROM ${command.relationName} ${pred};`;
}
exports.generateDelete = generateDelete;
function GenerateSqlRelationString(r, replace = false) {
    switch (r.relationType) {
        case "Table" /* Table */:
            return generateTableSpec(r, replace);
        case "View" /* View */:
            return generateSqlViews(r, replace);
        default:
            messages_1.LogInternalError("Not all Sql relation types are handled", messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
            return "";
    }
}
exports.GenerateSqlRelationString = GenerateSqlRelationString;
// FIXME note that we should probably not use the if not exist as a crutch
function generateTableSpec(t, replace = false) {
    const replaceQuery = replace ? `DROP TABLE IF EXISTS ${t.rName};` : "";
    return `${replaceQuery}
  CREATE TABLE ${t.rName} (
${t.columns.map(c => "    " + generateColumnDefinition(c)).join(",\n")}
  )`;
}
// helper function for quick DIEL code inspection
function GenerateStrFromDielDerivedRelation(v) {
    const rStr = v.relationType === "EventView" /* EventView */
        ? "event view"
        : v.relationType === "Output" /* Output */
            ? "output"
            : "view";
    return `create ${rStr} ${v.rName} AS ${generateSelect(v.selection.compositeSelections)}`;
}
exports.GenerateStrFromDielDerivedRelation = GenerateStrFromDielDerivedRelation;
function generateSqlViews(v, replace = false) {
    const replaceQuery = replace ? `DROP VIEW IF EXISTS ${v.rName};` : "";
    return `${replaceQuery}
  CREATE VIEW ${v.rName} AS
  ${generateSelect(v.selection)}
  `;
}
exports.generateSqlViews = generateSqlViews;
function generateSelect(v) {
    return v.map(c => GetSqlStringFromCompositeSelectionUnit(c)).join("\n");
}
exports.generateSelect = generateSelect;
const setOperatorToString = new Map([
    [dielAstTypes_1.SetOperator.NA, ""],
    [dielAstTypes_1.SetOperator.UNION, "UNION"],
    // [SetOperator.EXCEPT, "EXCEPT"],
    // [SetOperator.UNIONALL, "UNION ALL"],
    [dielAstTypes_1.SetOperator.INTERSECT, "INTERSECT"],
]);
function GetSqlStringFromCompositeSelectionUnit(c) {
    const op = setOperatorToString.get(c.op);
    const query = SqlStrFromSelectionUnit(c.relation);
    // the replace is a temporay patch to make the results look better
    return `${op} ${query}`.replace(/  \n[  \n]+/g, " ");
}
exports.GetSqlStringFromCompositeSelectionUnit = GetSqlStringFromCompositeSelectionUnit;
function SqlStrFromSelectionUnit(v) {
    // sometimes might want to use the un-normalized case for a quick SQL string...
    const selection = v.derivedColumnSelections
        ? generateColumnSelection(v.derivedColumnSelections)
        : generateColumnSelection(v.columnSelections);
    return `SELECT ${v.isDistinct ? "DISTINCT" : ""} ${selection}
    ${SqlStrFromSelectionUnitBody(v)}
  `;
}
exports.SqlStrFromSelectionUnit = SqlStrFromSelectionUnit;
/**
 * this is exported so that the codeDiv can use it
 * need to be cleaner for the future
 * @param v
 */
function SqlStrFromSelectionUnitBody(v) {
    const w = (v.whereClause) ? generateWhere(v.whereClause) : "";
    return `${v.baseRelation ? `FROM ${sqlStrFromRelationReference(v.baseRelation)}` : ""}
  ${v.joinClauses ? v.joinClauses.map(j => generateJoin(j)).join("\n") : ""}
  ${w}
  ${generateGroupBy(v.groupByClause)}
  ${generateOrderBy(v.orderByClause)}
  ${generateLimit(v.limitClause)}`;
}
exports.SqlStrFromSelectionUnitBody = SqlStrFromSelectionUnitBody;
function sqlStrFromRelationReference(ref) {
    // here there will be no stars..
    let query = "";
    switch (ref.relationReferenceType) {
        case dielAstTypes_1.RelationReferenceType.Direct: {
            const r = ref;
            query += r.relationName;
            break;
        }
        case dielAstTypes_1.RelationReferenceType.Subquery: {
            const r = ref;
            query += `(${generateSelect(r.subquery.compositeSelections)})`;
            break;
        }
        default:
            messages_1.LogInternalError(`RelationReference ${ref.relationReferenceType} not handled`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
    }
    if (ref.alias) {
        query += ` AS ${ref.alias}`;
    }
    return query;
}
/**
 * Note that here we will assume there is no
 * @param s
 */
function generateColumnSelection(s) {
    if (!s || s.length === 0) {
        return "";
    }
    return `${s.map(c => {
        const alias = c.alias ? ` AS ${c.alias}` : "";
        return GetSqlStringFromExpr(c.expr) + alias;
    }).join(", ")}`;
}
const joinOpToString = new Map([
    [dielAstTypes_1.JoinType.LeftOuter, "LEFT OUTER JOIN"],
    [dielAstTypes_1.JoinType.Inner, "JOIN"],
    [dielAstTypes_1.JoinType.CROSS, "CROSS JOIN"],
    [dielAstTypes_1.JoinType.Natural, "NATURAL JOIN"],
]);
function generateJoin(j) {
    if (!j)
        return "";
    const op = joinOpToString.get(j.joinType);
    const pred = j.predicate
        ? `ON ${GetSqlStringFromExpr(j.predicate)}`
        : "";
    return `${op} ${sqlStrFromRelationReference(j.relation)} ${pred}`;
}
function generateWhere(e) {
    if (!e)
        return "";
    return `WHERE ${GetSqlStringFromExpr(e)}`;
}
// recursive fun...
function GetSqlStringFromExpr(e) {
    switch (e.exprType) {
        case dielAstTypes_1.ExprType.Val:
            const v = e;
            const str = v.value.toString();
            if ((e.dataType === dielAstTypes_1.DielDataType.String) || (e.dataType === dielAstTypes_1.DielDataType.TimeStamp)) {
                return `'${str}'`;
            }
            return str;
        case dielAstTypes_1.ExprType.Star: {
            const s = e;
            const prefix = s.relationName ? `${s.relationName}.` : "";
            return `${prefix}*`;
        }
        case dielAstTypes_1.ExprType.Column: {
            const c = e;
            const prefix = c.relationName ? `${c.relationName}.` : "";
            return `${prefix}${c.columnName}`;
        }
        case dielAstTypes_1.ExprType.Relation:
            const r = e;
            return `(${generateSelect(r.selection.compositeSelections)})`;
        case dielAstTypes_1.ExprType.Parenthesis:
            const p = e;
            return `(${GetSqlStringFromExpr(p.content)})`;
        case dielAstTypes_1.ExprType.Func:
            const f = e;
            // this is functions
            switch (f.functionType) {
                case dielAstTypes_1.FunctionType.Math:
                case dielAstTypes_1.FunctionType.Compare:
                case dielAstTypes_1.FunctionType.Logic:
                    // assert that there are two args
                    if (f.args.length !== 2) {
                        messages_1.ReportDielUserError(`Function ${f.functionReference} should have 2 arguemnts`);
                    }
                    return `${GetSqlStringFromExpr(f.args[0])} ${f.functionReference} ${GetSqlStringFromExpr(f.args[1])}`;
                case dielAstTypes_1.FunctionType.BuiltIn:
                    switch (f.functionReference) {
                        case dielAstTypes_1.BuiltInFunc.ConcatStrings:
                            return f.args.map(a => GetSqlStringFromExpr(a)).join(" || ");
                        case dielAstTypes_1.BuiltInFunc.IfThisThen:
                            const whenCond = GetSqlStringFromExpr(f.args[0]);
                            const thenExpr = GetSqlStringFromExpr(f.args[1]);
                            const elseExpr = GetSqlStringFromExpr(f.args[2]);
                            return `CASE WHEN ${whenCond} THEN ${thenExpr} ELSE ${elseExpr} END`;
                        case dielAstTypes_1.BuiltInFunc.ValueIsNotNull:
                        case dielAstTypes_1.BuiltInFunc.ValueIsNull:
                            // should only have one
                            if (f.args.length !== 1) {
                                messages_1.LogInternalError(`Is or Not Null filters should only have one argument`);
                            }
                            return `${GetSqlStringFromExpr(f.args[0])} ${f.functionReference}`;
                        default:
                            // the rest should work with their references
                            return `${f.functionReference} (${f.args.map(a => GetSqlStringFromExpr(a)).join(", ")})`;
                    }
                case dielAstTypes_1.FunctionType.Custom:
                    return `${f.functionReference} (${f.args.map(a => GetSqlStringFromExpr(a)).join(", ")})`;
                default:
                    messages_1.LogInternalError(`FunctionType type ${f.functionType} not handled`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
                    return null;
            }
        default:
            messages_1.LogInternalError(`Expr type ${e.exprType} not handled`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
            return null;
    }
}
exports.GetSqlStringFromExpr = GetSqlStringFromExpr;
function generateGroupBy(g) {
    if (!g)
        return "";
    const groups = g.selections.map(sI => GetSqlStringFromExpr(sI)).join(", ");
    if (g.predicate) {
        return `GROUP BY ${groups} HAVING ${GetSqlStringFromExpr(g.predicate)}`;
    }
    else {
        return `GROUP BY ${groups}`;
    }
}
function generateOrderBy(o) {
    if (!o)
        return "";
    const orders = o.map(i => `${GetSqlStringFromExpr(i.selection)} ${generateOrder(i.order)}`);
    return `ORDER BY ${orders.join(", ")}`;
}
function generateOrder(order) {
    if (!order)
        return "";
    return order === dielAstTypes_1.Order.ASC
        ? "ASC"
        : "DESC";
}
function generateLimit(e) {
    if (!e)
        return "";
    return `LIMIT ${GetSqlStringFromExpr(e)}`;
}
function generateTrigger({ trigger, replace = false }) {
    const replaceQuery = replace ? `DROP TRIGGER IF EXISTS ${trigger.tName};` : "";
    let program = `${replaceQuery}
  CREATE TRIGGER ${trigger.tName} AFTER INSERT ON ${trigger.afterRelationName}\nBEGIN\n`;
    program += trigger.commands.map(p => generateCommand(p)).join("\n");
    program += "\nEND;";
    return program;
}
function generateInsertClauseStringForValue(raw) {
    // it can be explicitly set to null, but not undefined
    if (raw === undefined) {
        messages_1.LogInternalError(`Insertion should be defined.`);
    }
    // need to avoid the case when 0 is entered, which is valid!
    return (raw === null)
        ? "null"
        : (typeof raw === "string") ? `'${raw}'` : raw;
}
exports.generateInsertClauseStringForValue = generateInsertClauseStringForValue;
function generateInserts(i) {
    if (!i)
        return "";
    const columns = i.columns && i.columns.length > 0
        ? `(${i.columns.map(c => c).join(", ")})`
        : "";
    const values = i.values
        ? `VALUES (${i.values.map(v => generateInsertClauseStringForValue(v)).join(", ")})`
        : i.selection
            ? generateSelect(i.selection.compositeSelections)
            : "";
    return `INSERT INTO ${i.relation} ${columns} ${values};`;
}
const TypeConversionLookUp = new Map([
    [dielAstTypes_1.DielDataType.String, "TEXT"],
    [dielAstTypes_1.DielDataType.Number, "REAL"],
    [dielAstTypes_1.DielDataType.Boolean, "INTEGER"],
    [dielAstTypes_1.DielDataType.TimeStamp, "DATETIME"]
]);
function generateColumnDefinition(c) {
    const typeStr = TypeConversionLookUp.get(c.dataType);
    if (!typeStr) {
        messages_1.LogInternalError(`data type ${c.dataType} is not mapped to string`);
    }
    const plainQuery = `${c.cName} ${typeStr}`;
    if (!c.constraints) {
        // LogInternalError(`Constraints for column ${c.name} is not defined`);
        return plainQuery;
    }
    const notNull = c.constraints.notNull ? "NOT NULL" : "";
    const unique = c.constraints.unique ? "UNIQUE" : "";
    const primary = c.constraints.primaryKey ? "PRIMARY KEY" : "";
    const autoincrement = c.constraints.autoincrement ? "AUTOINCREMENT" : "";
    const defaultVal = c.defaultValue ? `DEFAULT ${GetSqlStringFromExpr(c.defaultValue)}` : "";
    return `${plainQuery} ${notNull} ${unique} ${primary} ${autoincrement} ${defaultVal}`;
}
//# sourceMappingURL=codeGenSql.js.map