"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DielAstGetters_1 = require("./DielAstGetters");
const applyTemplate_1 = require("./passes/applyTemplate");
const dependency_1 = require("./passes/dependency");
const normalizeConstraints_1 = require("./passes/normalizeConstraints");
const normalizeColumnSelection_1 = require("./passes/normalizeColumnSelection");
const inferType_1 = require("./passes/inferType");
const DielAstVisitors_1 = require("./DielAstVisitors");
const normalizeAlias_1 = require("./passes/normalizeAlias");
const antlr4ts_1 = require("antlr4ts");
const parser = require("../parser/grammar/DIELParser");
const lexer = require("../parser/grammar/DIELLexer");
const generateAst_1 = require("../parser/generateAst");
function parse(code) {
    const inputStream = new antlr4ts_1.ANTLRInputStream(code);
    const l = new lexer.DIELLexer(inputStream);
    const tokenStream = new antlr4ts_1.CommonTokenStream(l);
    return new parser.DIELParser(tokenStream);
}
exports.parse = parse;
function ParsePlainSelectQueryAst(code) {
    const inputStream = new antlr4ts_1.ANTLRInputStream(code);
    const l = new lexer.DIELLexer(inputStream);
    const tokenStream = new antlr4ts_1.CommonTokenStream(l);
    const p = new parser.DIELParser(tokenStream);
    const tree = p.selectQuery();
    let visitor = new generateAst_1.default();
    let ast = visitor.visit(tree);
    return ast;
}
exports.ParsePlainSelectQueryAst = ParsePlainSelectQueryAst;
/**
 * this visits all the queries that are included in the string
 * @param code code string
 */
function ParsePlainDielAst(code) {
    // PrintCode(code);
    const inputStream = new antlr4ts_1.ANTLRInputStream(code);
    const l = new lexer.DIELLexer(inputStream);
    const tokenStream = new antlr4ts_1.CommonTokenStream(l);
    const p = new parser.DIELParser(tokenStream);
    const tree = p.queries();
    let visitor = new generateAst_1.default();
    let ast = visitor.visitQueries(tree);
    return ast;
}
exports.ParsePlainDielAst = ParsePlainDielAst;
/**
 * Note that the compilation here is logical
 * there will be another pass on how to do the phyical part
 * @param ir the IR that will be manipulated
 */
function CompileAst(ast) {
    applyTemplate_1.ApplyTemplates(ast);
    normalizeAlias_1.NormalizeAlias(ast);
    dependency_1.AddDepTree(ast);
    dependency_1.ArrangeInTopologicalOrder(ast);
    normalizeConstraints_1.NormalizeConstraints(ast);
    normalizeColumnSelection_1.NormalizeColumnSelection(ast);
    inferType_1.InferType(ast);
    return ast;
}
exports.CompileAst = CompileAst;
// there should be a progressive version of this already.
// do not support crossfilter for now, but easy to fix!
function CompileNewRelationToExistingAst(ast, relation) {
    if (DielAstGetters_1.IsRelationTypeDerived(relation.relationType)) {
        CompileDerivedAstGivenAst(ast, relation);
    }
    else {
        CompileOriginalAstGivenIr(ast, relation);
    }
}
exports.CompileNewRelationToExistingAst = CompileNewRelationToExistingAst;
function CompileDerivedAstGivenAst(ast, view) {
    DielAstVisitors_1.AddRelation(ast, view);
    applyTemplate_1.TryToApplyTemplate(view.selection);
    normalizeAlias_1.NormalizeAliasForDerivedRelation(view);
    dependency_1.AddSingleDependencyByDerivedRelation(ast, view);
    normalizeColumnSelection_1.NormalizeColumnForDerivedRelation(ast, view);
    inferType_1.InferTypeForDerivedRelation(ast, view);
    return view;
}
exports.CompileDerivedAstGivenAst = CompileDerivedAstGivenAst;
/**
 * TODO: we need to handled the original like derived if they have subqueries...
 * @param ast
 * @param original
 */
function CompileOriginalAstGivenIr(ast, original) {
    applyTemplate_1.TryToCopyRelationSpec(ast, original);
    // TODO: need to add the depends on dependency...
    normalizeConstraints_1.NormalizeConstraintsForSingleOriginalRelation(original);
    return original;
}
exports.CompileOriginalAstGivenIr = CompileOriginalAstGivenIr;
//# sourceMappingURL=compiler.js.map