"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dielAstTypes_1 = require("../../parser/dielAstTypes");
/**
 * Notes:
 * - We are assuming that crossfitlers do not have unions
 * @param ast DielAst
 */
function ApplyCrossfilter(ast) {
    const newSetsOfViews = ast.crossfilters.map(c => _getViews(c));
    ast.relations = ast.relations.concat(...newSetsOfViews);
}
exports.ApplyCrossfilter = ApplyCrossfilter;
function _getViews(xIr) {
    // we need to create different views
    // first the static ones
    const unfilteredViews = xIr.charts.map(c => {
        return {
            relationType: "View" /* View */,
            rName: `${c.chartName}Unfiltered`,
            selection: c.selection
        };
    });
    const filteredViews = xIr.charts.map(c => {
        const otherCharts = xIr.charts.filter(c2 => c2.chartName !== c.chartName);
        const base = c.selection.compositeSelections[0].relation;
        // might not be the most performant.
        let relation = JSON.parse(JSON.stringify(base));
        relation.joinClauses = base.joinClauses.concat(otherCharts.map(o => o.predicate));
        const selection = {
            astType: dielAstTypes_1.AstType.RelationSelection,
            compositeSelections: [{
                    op: dielAstTypes_1.SetOperator.NA,
                    relation
                }]
        };
        return {
            rName: `${c.chartName}Filtered`,
            relationType: "View" /* View */,
            selection
        };
    });
    // then the dynamic ones
    return unfilteredViews.concat(filteredViews);
}
//# sourceMappingURL=applyCrossfilter.js.map