"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DielAstGetters_1 = require("../DielAstGetters");
const messages_1 = require("../../util/messages");
const DielPhysicalExecution_1 = require("../DielPhysicalExecution");
const dielAstTypes_1 = require("../../parser/dielAstTypes");
// keeping this functional so we can test it properly, this is why
// this has so many parameters
function QueryDistributionRecursiveEval(distributions, scope, relationId) {
    // find where rel lives, need to access metadata, or just have it augmented with the metadata already?
    // base case
    const node = scope.augmentedDep.get(relationId);
    if (!node) {
        return messages_1.LogInternalError(`Relation ${relationId} not found!`);
    }
    const sharedPartialDistributionObj = {
        forRelationName: node.relationName,
        finalOutputName: scope.outputName,
    };
    // base case
    if (node.dependsOn.length === 0) {
        distributions.push(Object.assign({ relationName: node.relationName, from: node.remoteId, to: node.remoteId }, sharedPartialDistributionObj));
        if (!node.remoteId) {
            return messages_1.LogInternalError(`Node should be assigned here`);
        }
        return {
            relationName: node.relationName,
            dbId: node.remoteId,
            fromDbId: node.remoteId,
        };
    }
    // derived, need to look at the things it needs, then decide who should own this relation
    // logic that decides the relation
    const dependentRecResults = node.dependsOn
        .map(depRelation => QueryDistributionRecursiveEval(distributions, scope, depRelation));
    const owner = scope.selectRelationEvalOwner(new Set(dependentRecResults.filter(d => d).map(r => r.dbId)));
    // we need to check to see if we need to apply the default policy here
    dependentRecResults.map(result => {
        if (result) {
            distributions.push(Object.assign({ relationName: result.relationName, from: result.dbId, to: owner }, sharedPartialDistributionObj));
        }
    });
    const rType = scope.relationTypeLookup(relationId);
    if (rType === "EventView" /* EventView */ || rType === "Output" /* Output */) {
        // add an additional shipping
        distributions.push(Object.assign({ relationName: relationId, from: owner, to: DielPhysicalExecution_1.LocalDbId }, sharedPartialDistributionObj));
        return {
            relationName: node.relationName,
            dbId: DielPhysicalExecution_1.LocalDbId,
            fromDbId: owner,
        };
    }
    return {
        relationName: node.relationName,
        dbId: owner,
        fromDbId: owner,
    };
}
exports.QueryDistributionRecursiveEval = QueryDistributionRecursiveEval;
/**
 * there might be corner cases where the view is not shipped e.g. here V1 might not be shipped? it is
 *     V2
 *    / \
 *   V1  R
 *  / \
 * I   R
 */
function getShippingInfoFromDistributedEval() {
}
exports.getShippingInfoFromDistributedEval = getShippingInfoFromDistributedEval;
exports.EventTableColumns = [
    // timestamp is actually stored in all inputs...
    // {
    //   cName: BuiltInColumn.TIMESTAMP,
    //   dataType: DielDataType.TimeStamp,
    // },
    {
        cName: dielAstTypes_1.BuiltInColumn.TIMESTEP,
        dataType: dielAstTypes_1.DielDataType.Number,
    },
    {
        cName: dielAstTypes_1.BuiltInColumn.REQUEST_TIMESTEP,
        dataType: dielAstTypes_1.DielDataType.Number,
    }
];
exports.EventViewColumns = [
    {
        cName: dielAstTypes_1.BuiltInColumn.TIMESTEP,
        dataType: dielAstTypes_1.DielDataType.Number,
    },
    {
        cName: dielAstTypes_1.BuiltInColumn.REQUEST_TIMESTEP,
        dataType: dielAstTypes_1.DielDataType.Number,
    }
];
/**
 * creates a new column with a new object (NOT copied over)
 * @param selection
 */
function GetColumnsFromSelection(selection) {
    const originalColumns = selection[0].relation.derivedColumnSelections;
    if (!originalColumns) {
        return messages_1.LogInternalError(`query not normalized and cannot be distributed to main`);
    }
    const columns = originalColumns.map(c => {
        if (!c.alias)
            messages_1.LogInternalError(`Alias should be defined for selection:\n${JSON.stringify(selection)}`);
        return {
            cName: c.alias,
            dataType: c.expr.dataType,
        };
    });
    return columns;
}
exports.GetColumnsFromSelection = GetColumnsFromSelection;
function getEventViewCacheName(tableName) {
    return `${tableName}Cache`;
}
exports.getEventViewCacheName = getEventViewCacheName;
function getEventViewCacheReferenceName(tableName) {
    return `${tableName}Reference`;
}
exports.getEventViewCacheReferenceName = getEventViewCacheReferenceName;
function GetSqlOriginalRelationFromDielRelation(relation, addTimeColumns) {
    switch (relation.relationType) {
        // when we turn an event into a table, the table is dynamic!
        case "EventTable" /* EventTable */: {
            const i = relation;
            return {
                relationType: "Table" /* Table */,
                isDynamic: true,
                rName: i.rName,
                columns: addTimeColumns ? i.columns.concat(exports.EventTableColumns) : i.columns
            };
        }
        case "ExistingAndImmutable" /* ExistingAndImmutable */:
        case "Table" /* Table */: {
            const i = relation;
            return {
                relationType: "Table" /* Table */,
                rName: i.rName,
                columns: i.columns
            };
        }
        // when we turn a view into a table, the table is dynamic!
        case "Output" /* Output */:
        case "DerivedTable" /* DerivedTable */:
        case "View" /* View */:
        case "EventView" /* EventView */:
            const derived = relation;
            const originalColumns = GetColumnsFromSelection(derived.selection.compositeSelections);
            if (!originalColumns) {
                return messages_1.LogInternalError(`Original columns not defined for ${relation.rName}`);
            }
            let createSpec = {
                relationType: "Table" /* Table */,
                isDynamic: true,
                rName: relation.rName,
                columns: addTimeColumns ? originalColumns.concat(exports.EventViewColumns) : originalColumns
            };
            return createSpec;
        default:
            return messages_1.LogInternalError(`Should all be handled, but ${relation.relationType} was not, for ${relation.rName}`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
    }
}
exports.GetSqlOriginalRelationFromDielRelation = GetSqlOriginalRelationFromDielRelation;
function makeCacheJoinClause(cacheRelationName, referenceRelationName) {
    const dataId = "ORIGINAL_REQUEST_TIMESTEP";
    let returnVal = {
        astType: dielAstTypes_1.AstType.Join,
        joinType: dielAstTypes_1.JoinType.Inner,
        relation: {
            relationName: referenceRelationName,
            relationReferenceType: dielAstTypes_1.RelationReferenceType.Direct
        },
        predicate: {
            dataType: dielAstTypes_1.DielDataType.Boolean,
            exprType: dielAstTypes_1.ExprType.Func,
            functionType: dielAstTypes_1.FunctionType.Compare,
            functionReference: "=",
            args: [{
                    exprType: dielAstTypes_1.ExprType.Column,
                    relationName: cacheRelationName,
                    columnName: dataId,
                    dataType: dielAstTypes_1.DielDataType.Number,
                }, {
                    exprType: dielAstTypes_1.ExprType.Column,
                    relationName: referenceRelationName,
                    columnName: dataId,
                    dataType: dielAstTypes_1.DielDataType.Number,
                }
            ]
        }
    };
    return returnVal;
}
// [
//   {
//     cName: "TIMESTEP",
//     dataType: DielDataType.Number
//   }].concat(originalColumns)
// function getEventDerivedColumnSelections(
//   cacheTableName: string,
//   originalColumns:  Column[]
//   ): ColumnSelection[] {
//    return cols;
// }
function isCachable(relation) {
}
exports.isCachable = isCachable;
/**
 * Obtains the internal representation of a cached event view, consisting
 * of three relations: a cache table, containing the data normally contained
 * by the event view; a reference table, which maps request timesteps to its
 * corresponding data in the cache table; and a view sharing the name of the
 * event view, which joins the previous two tables to represent the final
 * behavior.
 *
 * @param relation the event view to be cached
 * @param addTimeColumns whether to add time columns.
 *        TODO: Ryan: Should always be true?
 */
function GetCachedEventView(relation, addTimeColumns) {
    const cacheName = getEventViewCacheName(relation.rName);
    const refName = getEventViewCacheReferenceName(relation.rName);
    const viewName = relation.rName;
    const originalColumns = GetColumnsFromSelection(relation.selection.compositeSelections);
    const cacheTableColumns = originalColumns.concat([
        {
            cName: "TIMESTEP",
            dataType: dielAstTypes_1.DielDataType.Number
        },
        {
            cName: "ORIGINAL_REQUEST_TIMESTEP",
            dataType: dielAstTypes_1.DielDataType.Number
        }
    ]);
    const cacheTable = {
        rName: cacheName,
        isDynamic: true,
        relationType: "Table" /* Table */,
        columns: cacheTableColumns
    };
    const cacheDielTable = {
        rName: cacheName,
        relationType: "EventTable" /* EventTable */,
        columns: cacheTableColumns
    };
    const referenceColumns = [
        {
            cName: "ORIGINAL_REQUEST_TIMESTEP",
            dataType: dielAstTypes_1.DielDataType.Number
        }, {
            cName: "REQUEST_TIMESTEP",
            dataType: dielAstTypes_1.DielDataType.Number
        }
    ];
    const referenceTable = {
        rName: refName,
        isDynamic: true,
        relationType: "Table" /* Table */,
        columns: referenceColumns
    };
    const referenceDielTable = {
        rName: refName,
        relationType: "Table" /* Table */,
        columns: referenceColumns
    };
    const columnSelections = [];
    originalColumns.forEach(function (c) {
        columnSelections.push({
            expr: {
                exprType: dielAstTypes_1.ExprType.Column,
                columnName: c.cName,
                relationName: cacheName,
                dataType: c.dataType,
            },
            alias: c.cName
        });
    });
    columnSelections.push({
        expr: {
            exprType: dielAstTypes_1.ExprType.Column,
            columnName: "TIMESTEP",
            relationName: cacheName,
            dataType: dielAstTypes_1.DielDataType.Number,
        },
        alias: "TIMESTEP"
    });
    columnSelections.push({
        expr: {
            exprType: dielAstTypes_1.ExprType.Column,
            columnName: "REQUEST_TIMESTEP",
            relationName: refName,
            dataType: dielAstTypes_1.DielDataType.Number,
        },
        alias: "REQUEST_TIMESTEP"
    });
    const compositeSelections = [{
            op: dielAstTypes_1.SetOperator.NA,
            relation: {
                baseRelation: {
                    relationName: cacheName,
                    relationReferenceType: dielAstTypes_1.RelationReferenceType.Direct
                },
                columnSelections,
                derivedColumnSelections: columnSelections,
                joinClauses: [makeCacheJoinClause(cacheName, refName)],
            }
        }];
    const view = {
        rName: viewName,
        relationType: "View" /* View */,
        selection: compositeSelections
    };
    const dielView = {
        rName: viewName,
        relationType: "View" /* View */,
        selection: {
            astType: dielAstTypes_1.AstType.RelationSelection,
            compositeSelections
        },
    };
    return {
        cacheTable,
        cacheDielTable,
        referenceTable,
        referenceDielTable,
        view,
        dielView
    };
}
exports.GetCachedEventView = GetCachedEventView;
/**
 * We need the relation, but also additional information about how it should be mapped.
 * @param relation
 */
function GetSqlDerivedRelationFromDielRelation(relation) {
    switch (relation.relationType) {
        case "EventTable" /* EventTable */:
        case "Table" /* Table */:
        case "ExistingAndImmutable" /* ExistingAndImmutable */:
            return messages_1.LogInternalError(`Cannot make table into views`);
        case "DerivedTable" /* DerivedTable */:
            return {
                rName: relation.rName,
                relationType: "Table" /* Table */,
                selection: relation.selection.compositeSelections
            };
        case "EventView" /* EventView */:
        case "Output" /* Output */:
        case "DerivedTable" /* DerivedTable */:
        case "EventView" /* EventView */:
        case "View" /* View */: {
            return {
                rName: relation.rName,
                relationType: "View" /* View */,
                selection: relation.selection.compositeSelections
            };
        }
        default:
            return messages_1.LogInternalError(`Should all be handled, but ${relation.relationType} was not`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
    }
}
exports.GetSqlDerivedRelationFromDielRelation = GetSqlDerivedRelationFromDielRelation;
// only create tables for what outputs depend on
// and intersect that too, just just onestep is fine
function findOutputDep(ast, depTree) {
    const outputDep = new Set();
    DielAstGetters_1.GetAllOutputs(ast).map(o => {
        const dep = depTree.get(o.rName);
        if (dep) {
            dep.dependsOn.map(d => outputDep.add(d));
        }
        else {
            messages_1.LogInternalError(`Dep for ${o.rName} not found`, messages_1.DielInternalErrorType.RelationNotFound);
        }
    });
    return outputDep;
}
exports.findOutputDep = findOutputDep;
//# sourceMappingURL=distributeQueries.js.map