"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dielAstTypes_1 = require("../../parser/dielAstTypes");
const messages_1 = require("../../util/messages");
const DielAstVisitors_1 = require("../DielAstVisitors");
function relationReferenceVisitor(r) {
    if (r.alias)
        return;
    switch (r.relationReferenceType) {
        case dielAstTypes_1.RelationReferenceType.Subquery:
            messages_1.ReportDielUserError(`Subqueries must be named!`);
            return;
        case dielAstTypes_1.RelationReferenceType.Direct:
            r.alias = r.relationName;
    }
}
function visitSelections(r) {
    r.columnSelections.map(c => {
        if (c.alias)
            return;
        c.alias = getAliasForExpr(c.expr);
    });
}
function NormalizeAliasForDerivedRelation(derived) {
    derived.selection.compositeSelections.map(c => {
        visitSelections(c.relation);
        if (c.relation.baseRelation) {
            relationReferenceVisitor(c.relation.baseRelation);
            if (c.relation.joinClauses) {
                c.relation.joinClauses.map(j => {
                    relationReferenceVisitor(j.relation);
                });
            }
        }
    });
}
exports.NormalizeAliasForDerivedRelation = NormalizeAliasForDerivedRelation;
/**
 * Need to normalize the names for the relation references
 * also for column names
 * @param ast
 */
function NormalizeAlias(ast) {
    // walk through all the Relation References
    DielAstVisitors_1.WalkThroughRelationReferences(ast, relationReferenceVisitor);
    DielAstVisitors_1.WalkThroughSelectionUnits(ast, visitSelections);
    return;
}
exports.NormalizeAlias = NormalizeAlias;
function getAliasForExpr(expr) {
    switch (expr.exprType) {
        case dielAstTypes_1.ExprType.Column:
            return expr.columnName;
        case dielAstTypes_1.ExprType.Func:
            messages_1.ReportDielUserWarning(`Should name func column!`);
            return expr.functionReference;
        case dielAstTypes_1.ExprType.Val:
            messages_1.ReportDielUserWarning(`Should name value column!`);
            const v = expr;
            return v.dataType + v.value;
        case dielAstTypes_1.ExprType.Star:
            return null;
        default:
            return messages_1.LogInternalError(`Not all ${expr.exprType} handled`, messages_1.DielInternalErrorType.UnionTypeNotAllHandled);
    }
}
//# sourceMappingURL=normalizeAlias.js.map