"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dielAstTypes_1 = require("../../../src/parser/dielAstTypes");
const dielAstTypes_2 = require("../../parser/dielAstTypes");
const DielAstVisitors_1 = require("../DielAstVisitors");
/**
 * This function traverses the places where `RelationReference` might be called
 *   e.g., DerivedRelations, SelectionUnits in programs etc.
 * @param ast
 */
function applyLatestToAst(ast) {
    // TODO: check
    DielAstVisitors_1.WalkThroughSelectionUnits(ast, applyLatestToSelectionUnit);
    // // first go through the derivedrelations
    // const derived = GetAllDerivedViews(ast);
    // derived.map(d => {d.selection.compositeSelections.map(c => {
    //   applyLatestToSelectionUnit(c.relation);
    // });
    // });
    // // also need to check programs and commands
    // ast.programs.forEach(c => {
    // if (c.astType === AstType.RelationSelection) {
    //   (c as RelationSelection).compositeSelections.map(c => {
    //   applyLatestToSelectionUnit(c.relation);
    //   });
    // }
    // });
}
exports.applyLatestToAst = applyLatestToAst;
function ApplyLatestToDerivedRelation(derived) {
    derived.selection.compositeSelections.map(s => applyLatestToSelectionUnit(s.relation));
    return derived;
}
exports.ApplyLatestToDerivedRelation = ApplyLatestToDerivedRelation;
// IN PLACE
function applyLatestToRelationReference(ref, selection) {
    if (ref.relationReferenceType === dielAstTypes_1.RelationReferenceType.Direct) {
        const r = ref;
        if (r.isLatest) {
            modifyWhereComplete(selection, r.relationName);
        }
    }
    return;
}
/**
 *  find all the RelationReference instances in the DerivedRelation ASTs
 *   check if they say "isLatest", turn that boolean into false, and change the subquery
 *
 *   report error if there is already a subquery --- LATEST can only be used with a simple
 *   named relation
 *
 *  note this will probably be recursive
 * @param relation
 */
function applyLatestToSelectionUnit(relation) {
    if (!relation.baseRelation)
        return;
    applyLatestToRelationReference(relation.baseRelation, relation);
    if (!relation.joinClauses)
        return;
    for (let i = 0; i < relation.joinClauses.length; i++) {
        applyLatestToRelationReference(relation.joinClauses[i].relation, relation);
    }
}
exports.applyLatestToSelectionUnit = applyLatestToSelectionUnit;
/**
 * Modify WhereClause in relation in place, appending timestep clause for the relationName.
 *
 * In other words, append the following to the ast whereclause:
 * relationName.timestep = (select max(relationName) from relationName).
 *
 * @param relation
 * @param relationName
 */
function modifyWhereComplete(relation, relationName) {
    const originalWhere = relation.whereClause;
    // create exprast for subquery (select max(timestep) from relation)
    const rhsExpr = createSubquery(relationName);
    const whereAST = {
        exprType: dielAstTypes_1.ExprType.Func,
        functionType: dielAstTypes_1.FunctionType.Logic,
        functionReference: "=",
        dataType: dielAstTypes_1.DielDataType.Boolean,
        args: []
    };
    // create exprast for relation.timestep
    let lhsExpr = {
        exprType: dielAstTypes_1.ExprType.Column,
        columnName: "timestep",
        relationName: relationName
    };
    if (originalWhere) {
        // Merge into a where query
        lhsExpr = modifyExistingWhere(originalWhere, lhsExpr);
    }
    // set the where clause in place
    whereAST.args = [lhsExpr, rhsExpr];
    relation.whereClause = whereAST;
}
/**
 * Modify existing whereClause and return it so that a new ExprAst can be appended
*/
function modifyExistingWhere(originalAST, lhs) {
    let andAst = {
        exprType: dielAstTypes_1.ExprType.Func,
        functionType: dielAstTypes_1.FunctionType.Logic,
        functionReference: "and",
        dataType: dielAstTypes_1.DielDataType.Boolean,
        args: [originalAST, lhs]
    };
    return andAst;
}
/**
 * Create ExprAst for the clause (select max(relationName) from relationName).
*/
function createSubquery(relationName) {
    const relationAST = {
        exprType: dielAstTypes_1.ExprType.Relation,
        dataType: dielAstTypes_1.DielDataType.Relation,
        selection: {
            astType: dielAstTypes_2.AstType.RelationSelection,
            compositeSelections: [
                {
                    op: dielAstTypes_1.SetOperator.NA,
                    relation: {
                        isDistinct: false,
                        derivedColumnSelections: [
                            {
                                alias: "timestep",
                                expr: {
                                    exprType: dielAstTypes_1.ExprType.Func,
                                    functionType: dielAstTypes_1.FunctionType.Custom,
                                    functionReference: "max",
                                    args: [
                                        {
                                            exprType: dielAstTypes_1.ExprType.Column,
                                            columnName: "timestep"
                                        }
                                    ]
                                }
                            }
                        ],
                        baseRelation: {
                            relationReferenceType: dielAstTypes_1.RelationReferenceType.Direct,
                            alias: null,
                            isLatest: false,
                            relationName: relationName
                        },
                    }
                }
            ]
        }
    };
    return relationAST;
}
//# sourceMappingURL=syntaxSugar.js.map