"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var DielDataType;
(function (DielDataType) {
    DielDataType["Void"] = "Void";
    DielDataType["String"] = "String";
    DielDataType["Number"] = "Number";
    DielDataType["TimeStamp"] = "TimeStamp";
    DielDataType["Boolean"] = "Boolean";
    DielDataType["Relation"] = "Relation";
})(DielDataType = exports.DielDataType || (exports.DielDataType = {}));
var StaticRelationType;
(function (StaticRelationType) {
    StaticRelationType["Local"] = "Local";
    StaticRelationType["WebWorker"] = "WebWorker";
    StaticRelationType["Server"] = "Server";
})(StaticRelationType = exports.StaticRelationType || (exports.StaticRelationType = {}));
var ProgramType;
(function (ProgramType) {
    ProgramType["Udf"] = "Udf";
    ProgramType["Insert"] = "Insert";
    // YIFAN TODO TODAY implement drop/delete
})(ProgramType = exports.ProgramType || (exports.ProgramType = {}));
var LoggingLevels;
(function (LoggingLevels) {
    LoggingLevels["Verbose"] = "Verbose";
    LoggingLevels["Succinct"] = "Succinct";
})(LoggingLevels = exports.LoggingLevels || (exports.LoggingLevels = {}));
exports.BuiltInColumnTyppes = [
    {
        column: "rowid",
        type: DielDataType.Number
    },
    {
        column: "timestep",
        type: DielDataType.Number
    },
    {
        column: "request_timestep",
        type: DielDataType.Number
    },
    {
        column: "timestamp",
        type: DielDataType.Number
    }
];
exports.BuiltInUdfTypes = [
    {
        udf: "DATETIME",
        type: DielDataType.TimeStamp,
    },
    {
        udf: "COUNT",
        type: DielDataType.Number
    },
    {
        udf: "MIN",
        type: DielDataType.Number
    },
    {
        udf: "MAX",
        type: DielDataType.Number
    },
    {
        udf: "SUM",
        type: DielDataType.Number
    },
    {
        udf: "/",
        type: DielDataType.Number
    },
    {
        udf: "*",
        type: DielDataType.Number
    },
    {
        udf: "+",
        type: DielDataType.Number
    },
    {
        udf: "-",
        type: DielDataType.Number
    },
    {
        udf: "ROUND",
        type: DielDataType.Number
    },
    {
        udf: "AVG",
        type: DielDataType.Number
    },
    {
        udf: "GROUP_CONCAT",
        type: DielDataType.String
    },
];
var RelationOrigin;
(function (RelationOrigin) {
    RelationOrigin["User"] = "User";
    RelationOrigin["DerivedFromCaching"] = "DerivedFromCaching";
    RelationOrigin["DerivedFromMaterialization"] = "DerivedFromMaterialization";
})(RelationOrigin = exports.RelationOrigin || (exports.RelationOrigin = {}));
function NewRelationConstraints() {
    return {
        relationNotNull: false,
        relationHasOneRow: false,
        primaryKey: [],
        notNull: [],
        uniques: [],
        exprChecks: [],
        foreignKeys: []
    };
}
exports.NewRelationConstraints = NewRelationConstraints;
function createEmptyDielAst() {
    const newAst = {
        relations: [],
        replacedRelations: [],
        programs: new Map(),
        commands: [],
        udfTypes: exports.BuiltInUdfTypes,
        depTree: new Map()
    };
    return newAst;
}
exports.createEmptyDielAst = createEmptyDielAst;
var JoinType;
(function (JoinType) {
    JoinType["LeftOuter"] = "LeftOuter";
    JoinType["Inner"] = "Inner";
    JoinType["Natural"] = "Natural";
    JoinType["CROSS"] = "Cross";
})(JoinType = exports.JoinType || (exports.JoinType = {}));
/**
 * NA is used fort he first relation
 */
var SetOperator;
(function (SetOperator) {
    SetOperator["NA"] = "NA";
    SetOperator["UNION"] = "UNION";
    // UNIONALL = "UNIONALL",
    SetOperator["INTERSECT"] = "INTERSECT";
    // EXCEPT = "EXCEPT"
})(SetOperator = exports.SetOperator || (exports.SetOperator = {}));
var AstType;
(function (AstType) {
    AstType["Drop"] = "Drop";
    AstType["Delete"] = "Delete";
    AstType["Insert"] = "Insert";
    AstType["Join"] = "Join";
    AstType["RelationSelection"] = "RelationSelection";
})(AstType = exports.AstType || (exports.AstType = {}));
var RelationReferenceType;
(function (RelationReferenceType) {
    RelationReferenceType["Direct"] = "Direct";
    RelationReferenceType["Subquery"] = "Subquery";
})(RelationReferenceType = exports.RelationReferenceType || (exports.RelationReferenceType = {}));
var Order;
(function (Order) {
    Order["ASC"] = "ASC";
    Order["DESC"] = "DESC";
})(Order = exports.Order || (exports.Order = {}));
var DropType;
(function (DropType) {
    DropType["Table"] = "Table";
    DropType["View"] = "View";
    DropType["Trigger"] = "Trigger";
    DropType["Constraint"] = "Constraint";
    DropType["Index"] = "Index";
})(DropType = exports.DropType || (exports.DropType = {}));
/**
 * Notes
 * - expression has to be a recursive dataype
 * - we allow for sets in the Expr to make the set-oriented functions expressible.
 */
var ExprType;
(function (ExprType) {
    ExprType["Func"] = "Func";
    ExprType["Val"] = "Val";
    ExprType["Column"] = "Column";
    ExprType["Star"] = "Star";
    ExprType["Relation"] = "Relation";
    ExprType["Parenthesis"] = "Parenthesis";
})(ExprType = exports.ExprType || (exports.ExprType = {}));
/**
 * note that the string names here are used directly to generate the SQL queries
 *   so change the names carefully..
 * must be all caps for some laziness reasons
 */
var BuiltInFunc;
(function (BuiltInFunc) {
    BuiltInFunc["In"] = "IN";
    BuiltInFunc["DateTime"] = "DATETIME";
    BuiltInFunc["JulianDay"] = "JULIANDAY";
    BuiltInFunc["Coalesce"] = "COALESCE";
    BuiltInFunc["ValueIsNull"] = "IS NULL";
    BuiltInFunc["ValueIsNotNull"] = "NOT NULL";
    BuiltInFunc["SetEmpty"] = "NOT EXIST";
    BuiltInFunc["SetNotEmpty"] = "EXIST";
    // specially parsed when SQL gen
    BuiltInFunc["IfThisThen"] = "IFTHISTHEN";
    BuiltInFunc["ConcatStrings"] = "CONCATSTRINGS";
})(BuiltInFunc = exports.BuiltInFunc || (exports.BuiltInFunc = {}));
var FunctionType;
(function (FunctionType) {
    FunctionType["Math"] = "Math";
    FunctionType["Compare"] = "Compare";
    FunctionType["Logic"] = "Logic";
    FunctionType["BuiltIn"] = "BuiltIn";
    FunctionType["Custom"] = "Custom";
})(FunctionType = exports.FunctionType || (exports.FunctionType = {}));
var BuiltInColumn;
(function (BuiltInColumn) {
    BuiltInColumn["TIMESTEP"] = "TIMESTEP";
    BuiltInColumn["TIMESTAMP"] = "TIMESTAMP";
    BuiltInColumn["REQUEST_TIMESTEP"] = "REQUEST_TIMESTEP";
})(BuiltInColumn = exports.BuiltInColumn || (exports.BuiltInColumn = {}));
exports.BuiltInColumnDataTypes = new Map([
    [BuiltInColumn.TIMESTAMP.toString(), DielDataType.TimeStamp],
    [BuiltInColumn.TIMESTEP.toString(), DielDataType.Number],
    [BuiltInColumn.REQUEST_TIMESTEP.toString(), DielDataType.Number],
]);
// export enum MathOp {
//   ADD,
//   SUB,
//   MUL,
//   DIV
// }
// export enum CompareOp {
//   EQ,
//   NE,
//   GT,
//   GTE,
//   LT,
//   LTE
// }
//# sourceMappingURL=dielAstTypes.js.map