import { AbstractParseTreeVisitor } from "antlr4ts/tree";
import * as parser from "./grammar/DIELParser";
import * as visitor from "./grammar/DIELVisitor";
import { ExpressionValue, DerivedRelation, Command, DielAst, UdfType, OriginalRelation, RelationConstraints, ProgramsParserIr, InsertionClause, DropClause, Column, RelationSelection, CompositeSelectionUnit, ColumnSelection, SelectionUnit, JoinAst, OrderByAst, RawValues, GroupByAst, DeleteClause, ExprAst, ExprValAst, ExprFunAst, ExprColumnAst, ExprParen, ExprRelationAst, ExprStarAst, RelationReferenceDirect, RelationReferenceSubquery } from "./dielAstTypes";
export default class Visitor extends AbstractParseTreeVisitor<ExpressionValue> implements visitor.DIELVisitor<ExpressionValue> {
    private ast;
    private templates;
    defaultResult(): string;
    getAndCopyTemplateAst(templateName: string): JoinAst | RelationSelection | null;
    setContext(ir: DielAst): void;
    visitQueries: (ctx: parser.QueriesContext) => DielAst;
    visitRegisterTypeUdf(ctx: parser.RegisterTypeUdfContext): UdfType;
    visitSelectQueryDirect(ctx: parser.SelectQueryDirectContext): RelationSelection;
    visitCompositeSelect(ctx: parser.CompositeSelectContext): CompositeSelectionUnit | null;
    visitDropQuery(ctx: parser.DropQueryContext): DropClause;
    visitDeleteStmt(ctx: parser.DeleteStmtContext): DeleteClause;
    visitViewStmt(ctx: parser.ViewStmtContext): DerivedRelation;
    visitSelectUnitQuery(ctx: parser.SelectUnitQueryContext): SelectionUnit;
    visitSelectColumnClause(ctx: parser.SelectColumnClauseContext): ColumnSelection;
    visitExprSimple(ctx: parser.ExprSimpleContext): ExprAst;
    visitExprNegate(ctx: parser.ExprNegateContext): ExprFunAst;
    visitUnitExprColumn(ctx: parser.UnitExprColumnContext): ExprColumnAst | ExprStarAst;
    visitUnitExprSubQuery(ctx: parser.UnitExprSubQueryContext): ExprRelationAst;
    visitUnitExprValue(ctx: parser.UnitExprValueContext): ExprValAst;
    visitValueNumber(ctx: parser.ValueNumberContext): ExprValAst;
    visitValueString(ctx: parser.ValueStringContext): ExprValAst;
    visitValueBoolean(ctx: parser.ValueBooleanContext): ExprValAst;
    visitExprParenthesis(ctx: parser.ExprParenthesisContext): ExprParen;
    visitExprConcat(ctx: parser.ExprConcatContext): ExprAst;
    visitExprFunction(ctx: parser.ExprFunctionContext): ExprFunAst | null;
    visitExprBinOp(ctx: parser.ExprBinOpContext): ExprFunAst;
    visitExprNotNull(ctx: parser.ExprNotNullContext): ExprFunAst;
    visitExprNull(ctx: parser.ExprNullContext): ExprFunAst;
    visitExprExist(ctx: parser.ExprExistContext): ExprFunAst;
    visitExprIn(ctx: parser.ExprInContext): ExprFunAst;
    visitExprWhen(ctx: parser.ExprWhenContext): ExprFunAst;
    visitJoinClauseBasic(ctx: parser.JoinClauseBasicContext): JoinAst;
    visitGroupByClause(ctx: parser.GroupByClauseContext): GroupByAst;
    visitHavingClause(ctx: parser.HavingClauseContext): ExprAst;
    visitLimitClause(ctx: parser.LimitClauseContext): ExprAst;
    visitOrderByClause(ctx: parser.OrderByClauseContext): OrderByAst[];
    visitOrderSpec(ctx: parser.OrderSpecContext): OrderByAst;
    visitWhereClause(ctx: parser.WhereClauseContext): ExprAst;
    visitTemplateStmt(ctx: parser.TemplateStmtContext): null;
    visitJoinClauseTemplate(ctx: parser.JoinClauseTemplateContext): JoinAst;
    visitSelectQueryTemplate(ctx: parser.SelectQueryTemplateContext): RelationSelection;
    visitVariableAssignment(ctx: parser.VariableAssignmentContext): string[] | null;
    visitTemplateQuery(ctx: parser.TemplateQueryContext): JoinAst | RelationSelection | null;
    visitRelationReferenceSimple(ctx: parser.RelationReferenceSimpleContext): RelationReferenceDirect;
    visitRelationReferenceSubQuery(ctx: parser.RelationReferenceSubQueryContext): RelationReferenceSubquery;
    visitOriginalTableStmt(ctx: parser.OriginalTableStmtContext): OriginalRelation;
    visitRelationDefintionDirect(ctx: parser.RelationDefintionDirectContext): OriginalRelation;
    visitRelationDefintionCopy(ctx: parser.RelationDefintionCopyContext): string;
    _processConstraintDefinitionHelper(ctxs: parser.ConstraintDefinitionContext[]): RelationConstraints;
    visitConstraintClause(ctx: parser.ConstraintClauseContext): RelationConstraints;
    visitColumnDefinition(ctx: parser.ColumnDefinitionContext): Column | null;
    visitProgramStmt(ctx: parser.ProgramStmtContext): ProgramsParserIr;
    visitProgramBody(ctx: parser.ProgramBodyContext): Command[];
    visitInsertQuery(ctx: parser.InsertQueryContext): InsertionClause;
    visitInsertBodyDirect(ctx: parser.InsertBodyDirectContext): RawValues;
    visitInsertBodySelect(ctx: parser.InsertBodySelectContext): RelationSelection;
}
//# sourceMappingURL=generateAst.d.ts.map