"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tree_1 = require("antlr4ts/tree");
const dielAstTypes_1 = require("./dielAstTypes");
const visitorHelper_1 = require("./visitorHelper");
const messages_1 = require("../util/messages");
class Visitor extends tree_1.AbstractParseTreeVisitor {
    constructor() {
        super(...arguments);
        // this is serving also as a constrcutor
        // look into if there is a cleaner way to do this
        this.visitQueries = (ctx) => {
            this.ast = dielAstTypes_1.createEmptyDielAst();
            // this.seenInserts = [];
            this.templates = new Map();
            ctx.templateStmt().map(e => this.visitTemplateStmt(e));
            this.ast.udfTypes = this.ast.udfTypes.concat(ctx.registerTypeUdf().map(e => this.visit(e)));
            // two kinds of specifications
            const originalRelations = ctx.originalTableStmt().map(e => (this.visitOriginalTableStmt(e)));
            const derivedRelations = ctx.viewStmt().map(e => this.visit(e));
            this.ast.relations = originalRelations.concat(derivedRelations);
            // programs
            const programs = ctx.programStmt().map(e => this.visit(e));
            // console.log("raw programs", JSON.stringify(programs, null, 2));
            // console.log("current seen", JSON.stringify(this.seenInserts));
            programs.map(p => {
                p.events.map(e => {
                    const eventProgram = this.ast.programs.get(e);
                    if (eventProgram) {
                        eventProgram.push(...p.queries);
                    }
                    else {
                        this.ast.programs.set(e, p.queries);
                    }
                });
            });
            // commands
            // make sure not to execute the inserts that have been executed in programs.
            const insert = ctx.insertQuery()
                // .filter(e => this.seenInserts.find(i => i === e.start.startIndex) === null)
                .map(e => this.visit(e));
            const drops = ctx.dropQuery().map(e => this.visit(e));
            const deletes = ctx.deleteStmt().map(e => this.visit(e));
            this.ast.commands = insert.concat(drops).concat(deletes);
            return this.ast;
        };
        // visitSelectionStmt(ctx: parser.SelectionStmtContext): UISelection {
        //   const cell = ctx._viewName.text;
        //   const sName = ctx._selectionName.text;
        //   return {
        //     cell,
        //     sName
        //   };
        // }
        // visitFilterStmt(ctx: parser.FilterStmtContext): UIFilter {
        //   const cell = ctx._viewName.text;
        //   const fName = ctx._filterName.text;
        //   const selection = ctx._selectionName.text;
        //   return {
        //     cell,
        //     fName,
        //     selection,
        //   };
        // }
        // visitCrossfilterStmt(ctx: parser.CrossfilterStmtContext): UICrossfilter {
        //   const crossfilterName = ctx._crossfilterName.text;
        //   const relation = ctx._relationName.text;
        //   const measure = ctx._measureColumnName.text;
        //   return {
        //     crossfilterName,
        //     relation,
        //     factors,
        //     measure,
        //   };
        // }
        // crossfilter
        // visitCrossfilterStmt(ctx: parser.CrossfilterStmtContext): CrossFilterIr | null {
        //   const crossfilter = ctx._crossfilterName.text;
        //   const relation = ctx._relation.text;
        //   const charts = ctx.crossfilterChartStmt().map(c => this.visit(c) as CrossFilterChartIr);
        //   if (crossfilter && relation) {
        //     return {
        //       crossfilter,
        //       relation,
        //       charts
        //     };
        //   }
        //   return ReportDielUserError(`Crosffilter must speficy`);
        // }
        // visitCrossfilterChartStmt(ctx: parser.CrossfilterChartStmtContext): CrossFilterChartIr | null {
        //   const chartName = ctx._chart.text;
        //   const selection = this.visit(ctx._definitionQuery) as RelationSelection;
        //   const predicate = this.visit(ctx._predicateClause) as JoinAst;
        //   if (chartName) {
        //     return {
        //       chartName,
        //       selection,
        //       predicate
        //     };
        //   }
        //   return ReportDielUserError(`Crosffilter must speficy chart name`);
        // }
    }
    // this is to avoid visiting commands that are defined in programs
    // must be a better programming pattern to deal with this, hack for now... :/
    // private seenInserts: number[];
    defaultResult() {
        messages_1.LogInternalError("All the visits should be handled");
        return "";
    }
    getAndCopyTemplateAst(templateName) {
        const tDef = this.templates.get(templateName);
        if (tDef) {
            let newAst = null;
            switch (tDef.ast.astType) {
                case dielAstTypes_1.AstType.Join:
                    newAst = JSON.parse(JSON.stringify(tDef.ast));
                    break;
                case dielAstTypes_1.AstType.RelationSelection:
                    newAst = JSON.parse(JSON.stringify(tDef.ast));
                    break;
                default:
                    messages_1.LogInternalError(`Other AstTypes are not supported for template`);
            }
            return newAst;
        }
        else {
            messages_1.LogInternalError(`Template ${templateName} not defined`);
            return null;
        }
    }
    // this is useful for compiling partial queries
    setContext(ir) {
        messages_1.LogInfo("setting context");
        this.ast = ir;
    }
    visitRegisterTypeUdf(ctx) {
        const udf = ctx.IDENTIFIER().text;
        const type = visitorHelper_1.parseColumnType(ctx.dataType().text);
        return {
            udf,
            type
        };
    }
    // visitSelectQuery(ctx: parser.SelectQueryContext): RelationSelection {
    //   return this.visit(ctx) as RelationSelection;
    //   // if (ctx.)
    //   // return this.visit(ctx.getRuleContext());
    // }
    visitSelectQueryDirect(ctx) {
        // this is lazy, assume union or intersection to a hve the same columns
        const firstQuery = this.visit(ctx.selectUnitQuery());
        const compositeSelections = ctx.compositeSelect().map(e => this.visit(e));
        return {
            astType: dielAstTypes_1.AstType.RelationSelection,
            compositeSelections: [{ op: dielAstTypes_1.SetOperator.NA, relation: firstQuery }, ...compositeSelections]
        };
    }
    visitCompositeSelect(ctx) {
        const relation = this.visit(ctx.selectUnitQuery());
        const rawOp = ctx.setOp().text.toLocaleUpperCase();
        const op = (rawOp === "UNION")
            ? dielAstTypes_1.SetOperator.UNION
            : (rawOp === "INTERSECT")
                ? dielAstTypes_1.SetOperator.INTERSECT
                : null;
        if (!op) {
            return messages_1.LogInternalError(`Shouldn't Happen!`);
        }
        return {
            op,
            relation
        };
    }
    visitDropQuery(ctx) {
        const dropName = ctx.IDENTIFIER().text;
        return {
            astType: dielAstTypes_1.AstType.RelationSelection,
            dropType: dielAstTypes_1.DropType.Table,
            dropName,
        };
    }
    visitDeleteStmt(ctx) {
        const relationName = ctx.IDENTIFIER().text;
        const expr = ctx.expr();
        const predicate = ctx.WHERE() && expr
            ? this.visit(expr)
            : undefined;
        return {
            astType: dielAstTypes_1.AstType.Delete,
            relationName,
            predicate
        };
    }
    visitViewStmt(ctx) {
        const name = ctx.IDENTIFIER().text;
        const relationType = ctx.VIEW()
            ? ctx.EVENT()
                ? "EventView" /* EventView */
                : "View" /* View */
            : ctx.TABLE()
                ? "DerivedTable" /* DerivedTable */
                : "Output" /* Output */;
        const cClause = ctx.constraintClause();
        const constraints = cClause ? this.visit(cClause) : dielAstTypes_1.NewRelationConstraints();
        const selection = this.visit(ctx.selectQuery());
        return {
            rName: name,
            relationType,
            constraints,
            selection
        };
    }
    // TODO: do some type checking/inference on the selected.
    visitSelectUnitQuery(ctx) {
        const columnSelections = ctx.selectColumnClause().map(s => this.visit(s));
        if (columnSelections.length < 1) {
            const selectQuery = ctx.selectColumnClause().map(s => visitorHelper_1.getCtxSourceCode(s)).join(", ");
            throw new Error(`There should be some column values in select, query is ${selectQuery}`);
        }
        let body = null;
        const isDistinct = ctx.DISTINCT() ? true : false;
        if (ctx.FROM()) {
            const bClause = ctx.relationReference();
            const baseRelation = (bClause) ? this.visit(bClause) : undefined;
            const joinClauses = ctx.joinClause().map(e => this.visit(e));
            const wClause = ctx.whereClause();
            const whereClause = wClause ? this.visit(wClause) : undefined;
            const gClause = ctx.groupByClause();
            const groupByClause = gClause ? this.visitGroupByClause(gClause) : undefined;
            const oClause = ctx.orderByClause();
            const orderByClause = oClause ? this.visitOrderByClause(oClause) : undefined;
            const lClause = ctx.limitClause();
            const limitClause = lClause ? this.visitLimitClause(lClause) : undefined;
            body = {
                baseRelation,
                joinClauses,
                whereClause,
                groupByClause,
                orderByClause,
                limitClause
            };
        }
        return Object.assign({ isDistinct,
            columnSelections }, body);
    }
    visitSelectColumnClause(ctx) {
        const expr = this.visit(ctx.expr());
        const aliasText = ctx.IDENTIFIER();
        const alias = aliasText ? aliasText.text : undefined;
        return {
            alias,
            expr
        };
    }
    // begin level exprSimple
    visitExprSimple(ctx) {
        return this.visit(ctx.unitExpr());
    }
    // going to limit negation to boolean expressions for now
    visitExprNegate(ctx) {
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            dataType: dielAstTypes_1.DielDataType.Boolean,
            functionType: dielAstTypes_1.FunctionType.Logic,
            functionReference: "NOT",
            args: [this.visit(ctx.expr())]
        };
    }
    visitUnitExprColumn(ctx) {
        const hasStar = ctx.STAR() ? true : false;
        const columnName = ctx._column ? ctx._column.text : undefined;
        const relationName = ctx._relation ? ctx._relation.text : undefined;
        if (hasStar) {
            return {
                exprType: dielAstTypes_1.ExprType.Star,
                relationName
            };
        }
        return {
            exprType: dielAstTypes_1.ExprType.Column,
            columnName,
            relationName,
        };
    }
    visitUnitExprSubQuery(ctx) {
        const selection = this.visit(ctx.selectQuery());
        return {
            exprType: dielAstTypes_1.ExprType.Relation,
            dataType: dielAstTypes_1.DielDataType.Relation,
            selection,
        };
    }
    visitUnitExprValue(ctx) {
        return this.visit(ctx.value());
    }
    visitValueNumber(ctx) {
        return {
            exprType: dielAstTypes_1.ExprType.Val,
            dataType: dielAstTypes_1.DielDataType.Number,
            value: Number(ctx.NUMBER().text)
        };
    }
    visitValueString(ctx) {
        // because string contains the ', we need to strip it
        const raw = ctx.STRING().text;
        return {
            exprType: dielAstTypes_1.ExprType.Val,
            dataType: dielAstTypes_1.DielDataType.String,
            value: raw.slice(1, raw.length - 1)
        };
    }
    visitValueBoolean(ctx) {
        const value = (ctx.BOOLEANVAL().text === "true")
            ? true
            : false;
        return {
            exprType: dielAstTypes_1.ExprType.Val,
            dataType: dielAstTypes_1.DielDataType.String,
            value
        };
    }
    // begin paren
    visitExprParenthesis(ctx) {
        return {
            exprType: dielAstTypes_1.ExprType.Parenthesis,
            content: this.visit(ctx.expr())
        };
    }
    // begin
    visitExprConcat(ctx) {
        const args = ctx.expr().map(e => this.visit(e));
        return {
            exprType: dielAstTypes_1.ExprType.Val,
            dataType: dielAstTypes_1.DielDataType.String,
            functionType: dielAstTypes_1.FunctionType.BuiltIn,
            functionReference: dielAstTypes_1.BuiltInFunc.ConcatStrings,
            args
        };
    }
    visitExprFunction(ctx) {
        const functionReference = ctx._function.text;
        const args = ctx.expr().map(e => this.visit(e));
        // think about a more elegant comparison
        if (!functionReference) {
            return messages_1.ReportDielUserError(`Function name must be specified`);
        }
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            functionType: dielAstTypes_1.FunctionType.Custom,
            functionReference,
            args
        };
    }
    // begin level
    visitExprBinOp(ctx) {
        let functionType = dielAstTypes_1.FunctionType.Logic;
        let functionReference;
        let dataType = dielAstTypes_1.DielDataType.Boolean;
        if (ctx.mathOp()) {
            functionType = dielAstTypes_1.FunctionType.Math;
            functionReference = visitorHelper_1.getCtxSourceCode(ctx.mathOp());
            dataType = dielAstTypes_1.DielDataType.Number;
        }
        else if (ctx.compareOp()) {
            dielAstTypes_1.FunctionType.Compare;
            functionReference = visitorHelper_1.getCtxSourceCode(ctx.compareOp());
        }
        else {
            functionReference = visitorHelper_1.getCtxSourceCode(ctx.logicOp());
        }
        const args = ctx.expr().map(e => this.visit(e));
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            functionType,
            functionReference,
            dataType,
            args
        };
    }
    visitExprNotNull(ctx) {
        // const functionReference = ctx.NOT() ?  : BuiltInFunc.ValueIsNull;
        const arg = this.visit(ctx.expr());
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            dataType: dielAstTypes_1.DielDataType.Boolean,
            functionType: dielAstTypes_1.FunctionType.BuiltIn,
            functionReference: dielAstTypes_1.BuiltInFunc.ValueIsNotNull,
            args: [arg]
        };
    }
    // begin
    visitExprNull(ctx) {
        // const functionReference = ctx.NOT() ? BuiltInFunc.ValueIsNotNull : ;
        const arg = this.visit(ctx.expr());
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            dataType: dielAstTypes_1.DielDataType.Boolean,
            functionType: dielAstTypes_1.FunctionType.BuiltIn,
            functionReference: dielAstTypes_1.BuiltInFunc.ValueIsNull,
            args: [arg]
        };
    }
    visitExprExist(ctx) {
        const functionReference = ctx.NOT() ? dielAstTypes_1.BuiltInFunc.SetNotEmpty : dielAstTypes_1.BuiltInFunc.SetEmpty;
        const arg = this.visit(ctx.expr());
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            dataType: dielAstTypes_1.DielDataType.Boolean,
            functionType: dielAstTypes_1.FunctionType.BuiltIn,
            functionReference,
            args: [arg]
        };
    }
    visitExprIn(ctx) {
        const functionReference = dielAstTypes_1.BuiltInFunc.In;
        const args = ctx.expr();
        const arg1 = this.visit(args[0]);
        const arg2 = this.visit(args[1]);
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            dataType: dielAstTypes_1.DielDataType.Boolean,
            functionType: dielAstTypes_1.FunctionType.BuiltIn,
            functionReference,
            args: [arg1, arg2]
        };
    }
    visitExprWhen(ctx) {
        const func = dielAstTypes_1.BuiltInFunc.IfThisThen;
        const args = ctx.expr().map(e => this.visit(e));
        return {
            exprType: dielAstTypes_1.ExprType.Func,
            functionType: dielAstTypes_1.FunctionType.BuiltIn,
            functionReference: func,
            args
        };
    }
    visitJoinClauseBasic(ctx) {
        const relation = this.visit(ctx.relationReference());
        const pExpr = ctx.expr();
        const predicate = pExpr
            ? this.visit(pExpr)
            : undefined;
        const joinType = ctx.LEFT()
            ? dielAstTypes_1.JoinType.LeftOuter
            : ctx.NATURAL()
                ? dielAstTypes_1.JoinType.Natural
                : ctx.JOIN()
                    ? dielAstTypes_1.JoinType.Inner
                    : dielAstTypes_1.JoinType.CROSS;
        return {
            astType: dielAstTypes_1.AstType.Join,
            joinType,
            relation,
            predicate
        };
    }
    visitGroupByClause(ctx) {
        const selections = ctx.expr().map(e => this.visit(e));
        const hClause = ctx.havingClause();
        const predicate = hClause ? this.visitHavingClause(hClause) : undefined;
        return {
            selections,
            predicate
        };
    }
    visitHavingClause(ctx) {
        return this.visit(ctx.expr());
    }
    visitLimitClause(ctx) {
        return this.visit(ctx.expr());
    }
    visitOrderByClause(ctx) {
        return ctx.orderSpec().map(s => this.visitOrderSpec(s));
    }
    visitOrderSpec(ctx) {
        const order = ctx.DESC ? dielAstTypes_1.Order.DESC : dielAstTypes_1.Order.ASC;
        return {
            selection: this.visit(ctx.expr()),
            order
        };
    }
    visitWhereClause(ctx) {
        return this.visit(ctx.expr());
    }
    // template basically saves the AST and the AST gets parsed again when evaluated here
    visitTemplateStmt(ctx) {
        // not going to return anything, modify global
        const templateName = ctx._templateName.text;
        if (!templateName) {
            return messages_1.ReportDielUserError(`Template name must be specified`);
        }
        const variables = ctx.IDENTIFIER().map(i => i.text).splice(1);
        const sQuery = ctx.selectQuery();
        const jQuery = ctx.joinClause();
        if (sQuery) {
            const ast = this.visit(sQuery);
            this.templates.set(templateName, {
                variables,
                ast
            });
        }
        else if (jQuery) {
            const ast = this.visit(jQuery);
            this.templates.set(templateName, {
                variables,
                ast
            });
        }
        return null;
    }
    visitJoinClauseTemplate(ctx) {
        return this.visit(ctx.templateQuery());
    }
    visitSelectQueryTemplate(ctx) {
        return this.visit(ctx.templateQuery());
    }
    visitVariableAssignment(ctx) {
        const str = ctx._assignment.text;
        if ((!ctx._variable.text) || !str) {
            return messages_1.ReportDielUserError(`Variable name must be specified`);
        }
        return [ctx._variable.text, str.slice(1, str.length - 1)];
    }
    visitTemplateQuery(ctx) {
        const templateName = ctx._templateName.text;
        if (!templateName) {
            return messages_1.ReportDielUserError(`Template name must be specified`);
        }
        const templateSpec = new Map(ctx.variableAssignment().map(v => this.visit(v)));
        const templateAst = this.getAndCopyTemplateAst(templateName);
        if (!templateAst) {
            return messages_1.ReportDielUserError(`Template ${templateName} must be defined`);
        }
        templateAst.templateSpec = templateSpec;
        return templateAst;
    }
    visitRelationReferenceSimple(ctx) {
        const relationName = ctx._relation.text;
        const isLatest = ctx.LATEST() ? true : false;
        // check if the name is a relation
        const alias = ctx._alias ? ctx._alias.text : undefined;
        return {
            relationReferenceType: dielAstTypes_1.RelationReferenceType.Direct,
            alias,
            isLatest,
            relationName
        };
    }
    visitRelationReferenceSubQuery(ctx) {
        const subquery = this.visit(ctx.selectQuery());
        const alias = ctx._alias ? ctx._alias.text : undefined;
        // console.log(q);
        return {
            wasFromLatest: false,
            relationReferenceType: dielAstTypes_1.RelationReferenceType.Subquery,
            alias,
            subquery
        };
    }
    // visitStaticTableStmt(ctx: parser.StaticTableStmtContext): DerivedRelation {
    //   // it would not be the case of server here.
    //   const name = ctx.IDENTIFIER().text;
    //   const relationType = RelationType.StaticTable;
    //   const selection = this.visit(ctx.selectQuery()) as RelationSelection;
    //   return {
    //     name,
    //     relationType,
    //     selection
    //   };
    // }
    visitOriginalTableStmt(ctx) {
        if (ctx.EVENT() && ctx.REGISTER()) {
            messages_1.ReportDielUserError(`You cannot register an input relation`);
        }
        const relationType = ctx.EVENT()
            ? "EventTable" /* EventTable */
            : ctx.CREATE()
                ? "Table" /* Table */
                : "ExistingAndImmutable" /* ExistingAndImmutable */;
        const name = ctx.IDENTIFIER().text;
        let columns = [];
        const v = this.visit(ctx.relationDefintion());
        if (typeof v === "string") {
            const copyFrom = v;
            return {
                constraints: dielAstTypes_1.NewRelationConstraints(),
                relationType,
                rName: name,
                columns,
                copyFrom
            };
        }
        else {
            columns = v.columns;
            const constraints = v.constraints;
            return {
                relationType,
                rName: name,
                columns,
                constraints,
            };
        }
    }
    visitRelationDefintionDirect(ctx) {
        const columns = ctx.columnDefinition()
            ? ctx.columnDefinition().map(e => this.visit(e))
            : [];
        const constraints = this._processConstraintDefinitionHelper(ctx.constraintDefinition());
        // name and relationType are dummy to avoid having to add more to the union type...
        return {
            rName: "",
            relationType: "Table" /* Table */,
            columns,
            constraints
        };
    }
    visitRelationDefintionCopy(ctx) {
        return ctx.IDENTIFIER().text;
    }
    _processConstraintDefinitionHelper(ctxs) {
        let primaryKey = [];
        let uniques = [];
        let exprChecks = [];
        let notNull = [];
        let foreignKeys = [];
        ctxs.map(e => {
            const expr = e.expr();
            if (e.PRIMARY()) {
                primaryKey = e.IDENTIFIER().map(i => i.text);
            }
            else if (e.UNIQUE()) {
                const aUnique = e.IDENTIFIER().map(i => i.text);
                uniques.push(aUnique);
            }
            else if (e.NULL()) {
                notNull.push(e.IDENTIFIER()[0].text);
            }
            else if (e.CHECK() && expr) {
                exprChecks.push(this.visit(expr));
            }
            else if (e.FOREIGN()) {
                const sourceColumn = e._column.text;
                const targetRelation = e._table.text;
                const targetColumn = e._otherColumn.text;
                if (sourceColumn && targetRelation && targetColumn) {
                    foreignKeys.push({
                        sourceColumn,
                        targetRelation,
                        targetColumn,
                    });
                }
                else {
                    messages_1.ReportDielUserError(`Foreign keys must be defined`);
                }
            }
        });
        return {
            relationNotNull: false,
            relationHasOneRow: false,
            primaryKey,
            notNull,
            uniques,
            exprChecks,
            foreignKeys
        };
    }
    visitConstraintClause(ctx) {
        return this._processConstraintDefinitionHelper(ctx.constraintDefinition());
    }
    visitColumnDefinition(ctx) {
        let constraints = {
            autoincrement: false,
            notNull: false,
            unique: false,
            primaryKey: false
        };
        ctx.columnConstraints().map(e => {
            if (e.UNIQUE()) {
                constraints.unique = true;
            }
            if (e.KEY()) {
                constraints.primaryKey = true;
            }
            if (e.NULL()) {
                constraints.notNull = true;
            }
            if (e.AUTOINCREMENT()) {
                constraints.autoincrement = true;
            }
        });
        let defaultValue = null;
        if (ctx.DEFAULT()) {
            if (ctx._function) {
                const functionReference = ctx._function.text;
                const args = ctx.value().map(e => this.visit(e));
                // think about a more elegant comparison
                defaultValue = {
                    exprType: dielAstTypes_1.ExprType.Func,
                    functionType: dielAstTypes_1.FunctionType.Custom,
                    functionReference,
                    args
                };
            }
            else {
                // this is a single value
                defaultValue = this.visit(ctx._singleValue);
            }
        }
        const cName = ctx._columnName.text;
        if (!cName) {
            return messages_1.ReportDielUserError(`Column name must be specified`);
        }
        return {
            cName,
            dataType: visitorHelper_1.parseColumnType(ctx.dataType().text),
            constraints,
            defaultValue
        };
    }
    // programs
    visitProgramStmt(ctx) {
        const events = ctx.IDENTIFIER().map(n => n.text);
        const queries = this.visit(ctx.programBody());
        return {
            events,
            queries
        };
    }
    // visitProgramStmtGeneral(ctx: parser.ProgramStmtGeneralContext): ProgramsParserIr {
    //   const queries = this.visit(ctx.programBody()) as Command[];
    //   return {
    //     input: null,
    //     queries
    //   };
    // }
    // visitProgramStmtSpecific(ctx: parser.ProgramStmtSpecificContext): ProgramsParserIr {
    //   const input = ctx.IDENTIFIER().text;
    //   const queries = this.visit(ctx.programBody()) as Command[];
    //   return {
    //     input,
    //     queries
    //   };
    // }
    visitProgramBody(ctx) {
        const programs = [];
        const rawPrograms = ctx.aProgram();
        for (let i = 0; i < rawPrograms.length; i++) {
            const e = rawPrograms[i];
            const iQuery = e.insertQuery();
            const sQuery = e.selectQuery();
            const dStmt = e.deleteStmt();
            if (iQuery) {
                programs.push(this.visit(iQuery));
            }
            else if (sQuery) {
                programs.push(this.visit(sQuery));
            }
            else if (dStmt) {
                programs.push(this.visit(dStmt));
            }
            else {
                messages_1.LogInternalError(`Unhandled command`);
            }
        }
        return programs;
    }
    visitInsertQuery(ctx) {
        const identifiers = ctx.IDENTIFIER().map(e => e.text);
        const relation = identifiers[0];
        // note that columsn might not be specified
        const columns = identifiers.slice(1);
        // just look ahead now
        const v = this.visit(ctx.insertBody());
        let selection;
        let values;
        if (Array.isArray(v)) {
            values = v;
        }
        else {
            selection = v;
        }
        return {
            astType: dielAstTypes_1.AstType.Insert,
            relation,
            columns,
            selection,
            values
        };
    }
    visitInsertBodyDirect(ctx) {
        return ctx.value().map(e => {
            const t = this.visit(e);
            return t.value;
        });
    }
    visitInsertBodySelect(ctx) {
        return this.visit(ctx.selectQuery());
    }
}
exports.default = Visitor;
//# sourceMappingURL=generateAst.js.map