import { DbType, RelationObject, DielRemoteMessage, DielRemoteReply, RemoteUpdateRelationMessage } from "./runtimeTypes";
import { RelationShippingFuncType } from "./DielRuntime";
import { DielPhysicalExecution } from "../compiler/DielPhysicalExecution";
import { ConnectionWrapper } from "./ConnectionWrapper";
import { LogicalTimestep, RelationNameType, DbIdType } from "../parser/dielAstTypes";
interface DbSetupConfigBase {
    dbType: DbType;
}
export interface SocketConfig extends DbSetupConfigBase {
    connection: string;
    message?: any;
}
export interface WorkerConfig extends DbSetupConfigBase {
    jsFile: string;
    dataFile: string;
}
export declare type DbSetupConfig = SocketConfig | WorkerConfig;
export default class DbEngine {
    totalMsgCount: number;
    msgCountTimeStamp: number;
    currentQueueHead: LogicalTimestep | null;
    physicalExeuctionRef: DielPhysicalExecution;
    queueMap: Map<LogicalTimestep, {
        deps: Set<RelationNameType>;
        received: Set<RelationNameType>;
        receivedValues: RemoteUpdateRelationMessage[];
        relationsToShip: Map<RelationNameType, Set<DbIdType>>;
    }>;
    config: DbSetupConfig;
    id: DbIdType;
    relationShippingCallback: RelationShippingFuncType;
    connection: ConnectionWrapper;
    constructor(config: DbSetupConfig, remoteId: number, relationShippingCallback: RelationShippingFuncType);
    Close(): void;
    setup(): Promise<DielRemoteReply>;
    private nextQueue;
    private evaluateQueueOnUpdateHandler;
    private extendMsgWithCustom;
    SendMsg(msg: DielRemoteMessage, isPromise?: boolean): Promise<DielRemoteReply> | null;
    /**
     * This function generates the handler for when the original messages (i.e., actions to the databases)
     *   have been completed.
     */
    getHandleMsgForRemote(): (msg: DielRemoteReply) => Promise<void>;
    private sanityCheck;
    setPhysicalExecutionReference(physicalExeuctionRef: DielPhysicalExecution): void;
    downloadDb(): void;
    /**
     * SqlitemasterQuery returns sql and name
     */
    getMetaData(id: DbIdType): Promise<{
        id: DbIdType;
        data: RelationObject | null;
    }>;
}
export {};
//# sourceMappingURL=DbEngine.d.ts.map