"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const antlr4ts_1 = require("antlr4ts");
// import initSqlJs from "sql.js";
const sql_js_1 = require("sql.js");
const DIELLexer_1 = require("../parser/grammar/DIELLexer");
const DIELParser_1 = require("../parser/grammar/DIELParser");
const runtimeTypes_1 = require("./runtimeTypes");
const dielAstTypes_1 = require("../parser/dielAstTypes");
const codeGenSql_1 = require("../compiler/codegen/codeGenSql");
const generateAst_1 = require("../parser/generateAst");
const compiler_1 = require("../compiler/compiler");
const dielUdfs_1 = require("../util/dielUdfs");
const dielUtils_1 = require("../util/dielUtils");
const messages_1 = require("../util/messages");
const runtimeHelper_1 = require("./runtimeHelper");
const DielPhysicalExecution_1 = require("../compiler/DielPhysicalExecution");
const DbEngine_1 = require("./DbEngine");
const generateViewConstraints_1 = require("../compiler/passes/generateViewConstraints");
const staticSql_1 = require("../compiler/codegen/staticSql");
const compiler_2 = require("../compiler/compiler");
const dependency_1 = require("../compiler/passes/dependency");
const DielAstGetters_1 = require("../compiler/DielAstGetters");
const distributeQueries_1 = require("../compiler/passes/distributeQueries");
// ugly global mutable pattern here...
exports.STRICT = false;
exports.LOGINFO = false;
// const locateFile = (pathname: any) => {
//   if (pathname === "sql-wasm.wasm") {
//     return require("../../node_modules/sql.js/dist/sql-wasm.wasm");
//   }
//   throw new Error(`Unhandled locate path: ${pathname}`);
// };
// const config = {
//   locateFile
// };
// FIXME: the new export pattern for sql.js is so weird.
// type Database = any;
// type Statement = any;
// type QueryResults = any;
exports.INIT_TIMESTEP = 1;
exports.SqliteMasterQuery = `
  SELECT sql, name
  FROM sqlite_master
  WHERE type='table'
    AND sql not null
    AND name != 'sqlite_sequence'`;
class ViewConstraintQuery {
}
class DielRuntime {
    constructor(config) {
        // verbose just to make sure that the exported type is kept in sync
        this.NewInputMany = (view, o, requestTimestep) => {
            requestTimestep = requestTimestep ? requestTimestep : this.timestep;
            this.newInputHelper(view, o, requestTimestep);
        };
        const startSetUpTime = performance.now();
        if (typeof window !== "undefined") {
            window.diel = this; // for debugging
        }
        // mutate global for logging
        exports.STRICT = config.isStrict ? config.isStrict : false;
        exports.LOGINFO = config.showLog ? config.showLog : false;
        if (!exports.LOGINFO) {
            console.log = () => { };
        }
        this.timestep = exports.INIT_TIMESTEP;
        this.config = config;
        this.eventByTimestep = new Map();
        this.visitor = new generateAst_1.default();
        this.constraintQueries = new Map();
        this.checkConstraints = true;
        this.dbEngines = new Map();
        this.runtimeOutputs = new Map();
        this.physicalMetaData = {
            dbs: new Map(),
            relationLocation: new Map(),
            cache: config.caching ? config.caching : false
        };
        this.staticRelationsSent = [];
        this.boundFns = [];
        this.runOutput = this.runOutput.bind(this);
        this.BindOutput = this.BindOutput.bind(this);
        this.setup(config.setupCb, startSetUpTime);
    }
    getEventByTimestep(timestep) {
        return this.eventByTimestep.get(timestep);
    }
    ShutDown() {
        this.db.close();
        this.dbEngines.forEach(e => {
            e.Close();
            // close
        });
    }
    /**
     * When the notebook invokes BindOutput it can invoke the data to be evaluated
     * @param outputName
     * @param reactFn
     * @param triggerEval
     */
    BindOutput(outputName, reactFn) {
        if (!this.runtimeOutputs.has(outputName)) {
            messages_1.ReportUserRuntimeError(`output not defined ${outputName}, from current outputs of: [${Array.from(this.runtimeOutputs.keys()).join(", ")}]`);
        }
        this.boundFns.push({ outputName: outputName, uiUpdateFunc: reactFn });
        // the goal here is to tug on all the events so that the event views that outputs depend on will be shared
        // without any initial interactions
        const staticTriggers = this.physicalExecution.getStaticAsyncViewTrigger(outputName);
        // const staticTriggers = SetDifference(new Set(staticTriggersRaw.map(t => t.relation)), );
        // static triggers should be done just once...
        if (staticTriggers && (staticTriggers.length > 0)) {
            messages_1.LogInfo(`Sending triggers for async static output: ${outputName}`, staticTriggers);
            for (let i = 0; i < staticTriggers.length; i++) {
                const t = staticTriggers[i];
                if (this.staticRelationsSent.find(s => (s.static === t.relation) && (s.output === outputName))) {
                    console.log("seen this before", t.relation, outputName);
                    continue;
                }
                const staticShip = this.physicalExecution.getBubbledUpRelationToShipForStatic(t.dbId, t.relation, outputName);
                console.log("staticShip result", staticShip);
                const rDb = this.findRemoteDbEngine(t.dbId);
                staticShip.map(t => {
                    const staticMsg = {
                        remoteAction: runtimeTypes_1.DielRemoteAction.ShipRelation,
                        relationName: t.relation,
                        dbId: t.destination,
                        requestTimestep: this.timestep,
                    };
                    rDb.SendMsg(staticMsg);
                });
                // const msg: RemoteShipRelationMessage = {
                //   remoteAction: DielRemoteAction.ShipRelation,
                //   relationName: t.relation,
                //   outputName: outputName,
                //   dbId: t.dbId,
                //   requestTimestep: INIT_TIMESTEP
                // };
                // const rDb = this.findRemoteDbEngine(t.dbId);
                // if (!rDb) return LogInternalError(`${t.dbId} not found`);
                // rDb.SendMsg(msg);
                this.staticRelationsSent.push({ static: t.relation, output: outputName });
            }
        }
        // then evaluate the local view
        this.runOutput({ outputName, uiUpdateFunc: reactFn });
    }
    /**
     * GetView need to know if it's dependent on an event table
     *   in which case it will go and fetch the depdent events which are NOT depent on inputs
     *   it will also remember if this was done before
     *   add these special cases into physical execution logic
     * @param view
     */
    GetView(view) {
        return this.simpleGetLocal(view);
    }
    NewInput(i, o, requestTimestep) {
        requestTimestep = requestTimestep ? requestTimestep : this.timestep;
        messages_1.LogInfo(`New input ${requestTimestep ? `at ${requestTimestep}` : ""}`, JSON.stringify(o));
        this.newInputHelper(i, [o], requestTimestep);
    }
    makeSubKey(r) {
        if (r.relationReferenceType === dielAstTypes_1.RelationReferenceType.Direct) {
            let relation = r.relationName;
            // we need to filter out the time related fields if they are not used
            let query = `select * from ${relation}`;
            console.log(`    Hashing "${query}" for caching`);
            return JSON.stringify(this.ExecuteStringQuery(query));
        }
        else {
            let subqry = r.subquery;
            let hash = "";
            subqry.compositeSelections.forEach(c => {
                let query = codeGenSql_1.GetSqlStringFromCompositeSelectionUnit(c);
                console.log(`    Hashing "${query}" for caching`);
                hash += JSON.stringify(this.ExecuteStringQuery(query));
            });
            return hash;
        }
    }
    /**
     * Improvements: could be smarter
     * @param eventView
     */
    getCacheKey(eventView) {
        let rDef = DielAstGetters_1.GetRelationDef(this.ast, eventView);
        let selections = [];
        let mainSelection = rDef.selection.compositeSelections[0].relation;
        // RYAN TODO: if baseRelation undefined? Also check in other places for this.
        selections.push(mainSelection.baseRelation);
        mainSelection.joinClauses.forEach(j => {
            selections.push(j.relation);
        });
        let key = "";
        selections.forEach(s => {
            if (DielPhysicalExecution_1.dependsOnLocalTables(dependency_1.getRelationReferenceDep(s), this.physicalMetaData.relationLocation)) {
                key += ";" + this.makeSubKey(s);
            }
        });
        return key;
    }
    /**
     * Returns the requestTimestep corresponding to the cached requestTimestep.
     * If cache miss, will return the requestTimestep passed in.
     * @param eventView
     * @param requestTimestep
     */
    getFromCacheOrMiss(eventView, requestTimestep) {
        let key = this.getCacheKey(eventView);
        let cachedRequestTimestep = this.cache.get(key);
        if (cachedRequestTimestep === undefined) {
            console.log(`    Cache miss for local dependencies of ${eventView}.`);
            this.cache.set(key, requestTimestep);
            return requestTimestep;
        }
        console.log(`    Cache hit for ${eventView}: cached requestTimestep is ${requestTimestep}`);
        return cachedRequestTimestep;
    }
    newInputHelper(eventName, objs, requestTimestep) {
        // start of tick
        this.timestep++;
        this.runtimeInputs.get("logTime").stmt.run({
            $timestep: this.timestep,
            $kind: "begin",
            $ts: performance.now()
        });
        this.eventByTimestep.set(this.timestep, eventName);
        // we should get the definition from the local SQL definitions...
        const caching = this.config.caching && this.GetRelationDef(eventName).relationType === "EventView" /* EventView */;
        const inputEvent = caching
            ? distributeQueries_1.getEventViewCacheName(eventName)
            : eventName;
        const builtIn = {
            TIMESTEP: this.timestep,
            REQUEST_TIMESTEP: requestTimestep,
            ORIGINAL_REQUEST_TIMESTEP: requestTimestep
        };
        // set the values
        objs.map(o => {
            const value = {};
            // note that we need to filter out the built in columns
            //   because they are provided by US, not the input object
            // filter(c => !(c in BuiltInColumn))
            this.runtimeInputs.get(inputEvent).columnNames.map(cName => {
                const builtInValue = builtIn[cName];
                if (builtInValue) {
                    value[`$${cName}`] = builtInValue;
                    return;
                }
                else {
                    const raw = o[cName];
                    // it can be explicitly set to null, but not undefined
                    if (raw === undefined) {
                        messages_1.ReportUserRuntimeError(`We expected the input ${cName}, but it was not defined for ${eventName} in the object ${JSON.stringify(objs, null, 2)}.`);
                    }
                    else {
                        value[`$${cName}`] = raw;
                    }
                    return;
                }
            });
            this.runtimeInputs.get(inputEvent).stmt.run(value);
        });
        this.runtimeInputs.get("allInput").stmt.run({
            $timestep: this.timestep,
            $inputRelation: inputEvent,
            $timestamp: Date.now(),
            $request_timestep: requestTimestep
        });
        // Factored out so we can repeat
        const relationsDependentOnCurrentEvent = dependency_1.DeriveDependentRelations(this.ast.depTree, eventName);
        if (!relationsDependentOnCurrentEvent) {
            return messages_1.LogInternalWarning(`Input ${eventName} as not dependencies`);
        }
        const notCachedViews = this.checkAndApplyCache(eventName, relationsDependentOnCurrentEvent, this.timestep);
        // must be ran after cache for cached views to be in effect for the current timestep
        this.runOutputsGivenNewInputForEvent(relationsDependentOnCurrentEvent);
        messages_1.LogExecutionTrace(`notCachedViews are`, JSON.stringify(notCachedViews));
        if (notCachedViews.length > 0) {
            this.shipWorkerInput(eventName, notCachedViews, this.timestep);
        }
        return 0;
    }
    runOutputsGivenNewInputForEvent(inputDep) {
        // there should be a step here to check for caching, because if it's cached it should be in the same timestep!
        this.boundFns.map(b => {
            if (inputDep.has(b.outputName)) {
                this.runOutput(b);
            }
        });
        this.runtimeInputs.get("logTime").stmt.run({
            $timestep: this.timestep,
            $kind: "end",
            $ts: performance.now()
        });
    }
    /**
     * this is the execution logic
     *   where we do distributed query execution
     *   and possibly materialization (basically can be in SQL triggers or our own even handling layer)
     */
    runOutput(b) {
        const r = this.simpleGetLocal(b.outputName);
        if (r) {
            b.uiUpdateFunc(r);
            this.constraintChecking(b.outputName);
        }
        return;
    }
    /**
     * Check view constraints afresh and report if broken
     */
    constraintChecking(viewName) {
        // TODO. Check view constraint for nested views. currently, it only checks output view.
        // only check if checking mode is turned on
        if (this.checkConstraints) {
            if (this.constraintQueries.has(viewName)) {
                let queryObject = this.constraintQueries.get(viewName);
                if (queryObject) {
                    let queries = queryObject.queries;
                    // run the entire constraint quries for that view
                    queries.map(ls => {
                        this.reportConstraintQueryResult(ls[0], viewName, ls[1]);
                    });
                }
            }
        }
    }
    downloadDB(dbId, fileName) {
        if ((!dbId) || (dbId === DielPhysicalExecution_1.LocalDbId)) {
            let dRaw = this.db.export();
            let blob = new Blob([dRaw]);
            const fName = fileName ? fileName : "session";
            dielUtils_1.downloadHelper(blob, fName);
        }
        else {
            const remote = this.dbEngines.get(dbId);
            if (remote) {
                remote.downloadDb();
            }
        }
    }
    simpleGetLocal(view) {
        const s = this.runtimeOutputs.get(view);
        if (s) {
            s.bind({});
            let r = [];
            while (s.step()) {
                r.push(s.getAsObject());
            }
            if (r.length > 0) {
                return r;
            }
            else {
                return messages_1.ReportUserRuntimeWarning(`${view} did not return any results.`);
            }
        }
        else {
            return messages_1.ReportUserRuntimeError(`${view} does not exist.`);
        }
        return null;
    }
    setup(loadPage, startSetUpTime) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(`Setting up DielRuntime with ${JSON.stringify(this.config)}`);
            yield this.setupMainDb();
            yield this.setupRemotes();
            yield this.initialCompile();
            this.setupUDFs();
            this.physicalExecution = new DielPhysicalExecution_1.DielPhysicalExecution(this.ast, this.physicalMetaData, this.getEventByTimestep.bind(this), this.AddRelation.bind(this));
            yield this.executeToDBs();
            this.setupNewInput();
            DielAstGetters_1.GetAllOutputs(this.ast).map(o => this.setupNewOutput(o.rName));
            this.scales = runtimeHelper_1.ParseSqlJsWorkerResult(this.db.exec("select * from __scales"));
            this.cache = new Map();
            const endSetUpTime = performance.now();
            this.runtimeInputs.get("logTime").stmt.run({
                $timestep: 0,
                $kind: "begin",
                $ts: startSetUpTime,
            });
            this.runtimeInputs.get("logTime").stmt.run({
                $timestep: 0,
                $kind: "end",
                $ts: endSetUpTime,
            });
            loadPage();
        });
    }
    initialCompile() {
        return __awaiter(this, void 0, void 0, function* () {
            this.visitor = new generateAst_1.default();
            const tableDefinitions = runtimeHelper_1.SqlJsGetObjectArrayFromQuery(this.db, exports.SqliteMasterQuery);
            tableDefinitions.map(m => {
                const name = m["name"].toString();
                if (this.physicalMetaData.relationLocation.has(name)) {
                    messages_1.LogInternalError(`DBs should not have the same table names`);
                }
                else {
                    this.physicalMetaData.relationLocation.set(name, {
                        dbId: DielPhysicalExecution_1.LocalDbId
                    });
                }
            });
            this.physicalMetaData.dbs.set(DielPhysicalExecution_1.LocalDbId, { dbType: runtimeTypes_1.DbType.Local });
            let code = runtimeHelper_1.processSqlMetaDataFromRelationObject(tableDefinitions, "main");
            const promises = [];
            this.dbEngines.forEach((db) => {
                this.physicalMetaData.dbs.set(db.id, { dbType: db.config.dbType });
                promises.push(db.getMetaData(db.id));
            });
            const metadatas = yield Promise.all(promises);
            // const allDielRelations = this.ir.GetAllRelationNames();
            metadatas.map(mD => {
                code += runtimeHelper_1.processSqlMetaDataFromRelationObject(mD.data, mD.id.toString());
                // .map(m => m["sql"] + ";").join("\n");
                mD.data.map(m => {
                    const name = m["name"].toString();
                    if (this.physicalMetaData.relationLocation.has(name)) {
                        messages_1.LogInternalError(`DBs should not have the same table names`);
                    }
                    else {
                        this.physicalMetaData.relationLocation.set(name, {
                            dbId: mD.id
                        });
                    }
                });
            });
            // get the static ones
            code += staticSql_1.StaticSql;
            for (let i = 0; i < this.config.dielFiles.length; i++) {
                const f = this.config.dielFiles[i];
                code += yield (yield fetch(f)).text();
            }
            messages_1.PrintCode(code);
            const inputStream = new antlr4ts_1.ANTLRInputStream(code);
            const p = new DIELParser_1.DIELParser(new antlr4ts_1.CommonTokenStream(new DIELLexer_1.DIELLexer(inputStream)));
            const tree = p.queries();
            let ast = this.visitor.visitQueries(tree);
            this.ast = compiler_1.CompileAst(ast);
            // get sql for views constraints
            generateViewConstraints_1.checkViewConstraint(ast).forEach((queries, viewName) => {
                if (queries.length > 0) {
                    let viewConstraint = new ViewConstraintQuery();
                    viewConstraint.viewName = viewName;
                    viewConstraint.queries = queries;
                    this.constraintQueries.set(viewName, viewConstraint);
                }
            });
            // test the IR here
        });
    }
    setupRemotes() {
        return __awaiter(this, void 0, void 0, function* () {
            const inputCallback = this.NewInputMany.bind(this);
            let counter = DielPhysicalExecution_1.LocalDbId;
            const dbWaiting = this.config.dbConfigs
                ? this.config.dbConfigs.map(config => {
                    counter++;
                    const remote = new DbEngine_1.default(config, counter, inputCallback);
                    this.dbEngines.set(counter, remote);
                    return remote.setup();
                })
                : [];
            // const workerWaiting = this.config.workerConfigs
            //   ? this.config.workerConfigs.map(config => {
            //     counter++;
            //     const remote = new DbEngine(DbType.Worker, counter, inputCallback);
            //     this.dbEngines.set(counter, remote);
            //     return remote.setup(config);
            //   })
            //   : [];
            // const socketWaiting = this.config.socketConfigs
            //   ? this.config.socketConfigs.map(config => {
            //     counter++;
            //     const remote = new DbEngine(DbType.Socket, counter, inputCallback);
            //     this.dbEngines.set(counter, remote);
            //     return remote.setup(config);
            //   })
            //   : [];
            yield Promise.all(dbWaiting);
            return;
        });
    }
    AddUDF(fName) {
        // a bit hacky
        this.db.create_function(fName, window[fName]);
    }
    setupUDFs() {
        this.db.create_function("log", dielUdfs_1.log);
    }
    /**
     * returns false if caching is not enabled or that it's a cache miss
     * - if the cache misses, then ship to everywhere
     * - if cache does not miss, then insert into reference directly (not an event)
     * -   we have to do this for every view because the same view might be triggered by multiple inputs
     * @param inputName
     * @param timestep
     */
    checkAndApplyCache(eventName, relationsDependentOnCurrentEvent, timestep) {
        const eventViews = [];
        relationsDependentOnCurrentEvent.forEach(dep => {
            const isEventView = DielPhysicalExecution_1.dependsOnRemoteTables(this.ast.depTree.get(dep).dependsOn, this.physicalMetaData.relationLocation);
            if (isEventView)
                eventViews.push(dep);
        });
        if (!this.config.caching)
            return eventViews;
        console.log(`Caching enabled, so determining whether we need to ship ${eventName}`);
        const notCachedViews = [];
        eventViews.map(eventView => {
            const requestTimestep = this.getFromCacheOrMiss(eventView, timestep);
            // we need to insert this into the cache table
            // but NOT advance the timestep
            const referenceRelation = distributeQueries_1.getEventViewCacheReferenceName(eventView);
            const query = `insert into ${referenceRelation} (REQUEST_TIMESTEP, ORIGINAL_REQUEST_TIMESTEP) values (${timestep}, ${requestTimestep})`;
            this.db.run(query);
            if (requestTimestep < timestep) {
                messages_1.LogExecutionTrace(`    Cached ${eventView}`);
            }
            else {
                // we have to wait for the result...
                notCachedViews.push(eventView);
                messages_1.LogExecutionTrace(`    NOT cached ${eventView}`);
            }
        });
        return notCachedViews;
    }
    /**
     * @param inputName
     * @param timestep
     */
    shipWorkerInput(inputName, notCachedViews, timestep) {
        const remotesToShipTo = this.physicalExecution.getBubbledUpRelationToShipForEvent(DielPhysicalExecution_1.LocalDbId, inputName, notCachedViews);
        // FIXME: we can improve performance by grouping by the views to ship so that they are not evaluated multiple times.
        if (remotesToShipTo && remotesToShipTo.length > 0) {
            remotesToShipTo.map(t => {
                const rDef = DielAstGetters_1.GetRelationDef(this.ast, t.relation);
                const columns = DielAstGetters_1.DeriveColumnsFromRelation(rDef);
                const shareQuery = `select ${columns.map(c => c.cName).join(", ")} from ${t.relation}`;
                let tableRes = this.db.exec(shareQuery)[0];
                if ((!tableRes) || (!tableRes.values)) {
                    messages_1.LogInternalWarning(`Query ${shareQuery} has NO result`);
                    return;
                }
                // FIXME: have more robust typing instead of quoting everything; look up types...
                const values = tableRes.values.map((d) => `(${d.map((v) => (v === null) ? "null" : `'${v}'`).join(", ")})`);
                let sql = `
          DELETE from ${t.relation};
          INSERT INTO ${t.relation} VALUES ${values};
        `;
                const updateMsg = {
                    remoteAction: runtimeTypes_1.DielRemoteAction.UpdateRelation,
                    relationName: t.relation,
                    requestTimestep: timestep,
                    sql
                };
                this.dbEngines.get(t.destination).SendMsg(updateMsg);
            });
        }
    }
    findRemoteDbEngine(remoteId) {
        const r = this.dbEngines.get(remoteId);
        if (!r)
            messages_1.LogInternalError(`Remote ${remoteId} not found`);
        return r;
    }
    /**
     * returns the DIEL code that will be ran to register the tables
     */
    setupMainDb() {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log("initSqlJs is:", initSqlJs);
            // const SQL = await initSqlJs(config);
            if (!this.config.mainDbPath) {
                this.db = new sql_js_1.Database();
            }
            else {
                const response = yield fetch(this.config.mainDbPath);
                const bufferRaw = yield response.arrayBuffer();
                const buffer = new Uint8Array(bufferRaw);
                this.db = new sql_js_1.Database(buffer);
            }
            return;
        });
    }
    /**
     * output tables HAVE to be local tables
     *   since they are synchronous --- if they are over other tables
     *   it would be handled via some trigger programs
     * FIXME: just pass in what it needs, the name str
     */
    setupNewOutput(rName) {
        console.log("setting up new output", rName);
        const q = `select * from ${rName}`;
        this.runtimeOutputs.set(rName, this.dbPrepare(q));
    }
    // what's dynamic is read from the local AST...
    setupNewInput() {
        this.runtimeInputs = new Map();
        const localSqlAst = this.physicalExecution.getAstFromDbId(DielPhysicalExecution_1.LocalDbId);
        const inputSqlAsts = localSqlAst.relations.filter(r => r.isDynamic);
        inputSqlAsts.map(eRaw => {
            const e = eRaw;
            const columnNames = e.columns.map(c => c.cName);
            const q = `insert into ${e.rName} values (${columnNames.map(c => `$${c}`).join(", ")})`;
            this.runtimeInputs.set(e.rName, {
                stmt: this.dbPrepare(q),
                columnNames
            });
        });
        this.runtimeInputs.set("allInput", {
            stmt: this.db.prepare(`insert into allInputs (timestep, inputRelation, timestamp, request_timestep) values ($timestep, $inputRelation, $timestamp, $request_timestep)`),
            columnNames: ["timestep", "inputRelation", "timestamp", "request_timestep"]
        });
        this.runtimeInputs.set("logTime", {
            stmt: this.db.prepare(`insert into __perf values ($timestep, $kind, $ts)`),
            columnNames: ["timestep", "kind", "ts"]
        });
    }
    dbPrepare(q) {
        try {
            return this.db.prepare(q);
        }
        catch (e) {
            messages_1.LogInternalError(`Got ${e} while preparing for query ${q}`);
            return null;
        }
    }
    /**
     * returns the results as an array of objects (sql.js)
     */
    ExecuteSqlAst(ast) {
        const queryString = codeGenSql_1.SqlStrFromSelectionUnit(ast);
        return this.ExecuteStringQuery(queryString);
    }
    ExecuteStringQuery(q) {
        let r = [];
        // fixme: better types
        this.db.each(q, (row) => { r.push(row); }, () => { });
        return r;
    }
    incrementalExecuteToDb(instructions) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [];
            instructions.map(i => {
                if (i.dbId === DielPhysicalExecution_1.LocalDbId) {
                    // const sqlAst = CreateUnitSqlFromUnitDiel(, false);
                    const sqlStr = codeGenSql_1.GenerateSqlRelationString(i.relationDef);
                    runtimeHelper_1.CaughtLocalRun(this.db, sqlStr);
                }
                else {
                    // FIXME: think about replacement logic
                    const sqlStr = codeGenSql_1.GenerateSqlRelationString(i.relationDef);
                    const remoteInstance = this.findRemoteDbEngine(i.dbId);
                    if (remoteInstance) {
                        // just to be sure
                        // FIXME: this is kinda redundant...
                        remoteInstance.setPhysicalExecutionReference(this.physicalExecution);
                        const msg = {
                            remoteAction: runtimeTypes_1.DielRemoteAction.DefineRelations,
                            requestTimestep: exports.INIT_TIMESTEP,
                            sql: sqlStr,
                        };
                        const mPromise = remoteInstance.SendMsg(msg, true);
                        promises.push(mPromise);
                    }
                }
            });
            yield Promise.all(promises);
            return;
        });
    }
    // takes in the SqlIrs in different environments and sticks them into the databases
    // FIXME: better async handling
    executeToDBs() {
        return __awaiter(this, void 0, void 0, function* () {
            messages_1.LogTmp(`Executing queries to db`);
            // now execute to worker!
            const promises = [];
            this.physicalExecution.sqlAstSpecPerDb.forEach((ast, id) => {
                if (id === DielPhysicalExecution_1.LocalDbId) {
                    const queries = codeGenSql_1.generateStringFromSqlIr(ast, false);
                    for (let s of queries) {
                        runtimeHelper_1.CaughtLocalRun(this.db, s);
                    }
                }
                else {
                    const remoteInstance = this.findRemoteDbEngine(id);
                    if (remoteInstance) {
                        remoteInstance.setPhysicalExecutionReference(this.physicalExecution);
                        const replace = remoteInstance.config.dbType === runtimeTypes_1.DbType.Socket;
                        const queries = codeGenSql_1.generateStringFromSqlIr(ast, replace);
                        if (queries && queries.length > 0) {
                            const sql = queries.map(q => q + ";").join("\n");
                            const msg = {
                                remoteAction: runtimeTypes_1.DielRemoteAction.DefineRelations,
                                requestTimestep: exports.INIT_TIMESTEP,
                                sql,
                            };
                            const mPromise = remoteInstance.SendMsg(msg, true);
                            if (mPromise)
                                promises.push(mPromise);
                        }
                        const deleteQueryAst = codeGenSql_1.generateCleanUpAstFromSqlAst(ast);
                        const deleteQueries = deleteQueryAst.map(d => codeGenSql_1.generateDrop(d)).join("\n");
                        console.log(`%c Cleanup queries:\n${deleteQueries}`, "color: blue");
                        if ((remoteInstance.config.dbType === runtimeTypes_1.DbType.Socket) && deleteQueries) {
                            const msg = {
                                remoteAction: runtimeTypes_1.DielRemoteAction.CleanUpQueries,
                                requestTimestep: exports.INIT_TIMESTEP,
                                sql: deleteQueries,
                            };
                            remoteInstance.SendMsg(msg, false);
                        }
                    }
                    else {
                        messages_1.LogInternalError(`Remote ${id} is not found!`);
                    }
                }
            });
            yield Promise.all(promises);
            return;
        });
    }
    // TODO: use component to do namespacing
    // this should really not be part of the runtime --- should more be part of DIEL-UI?
    GetScales(output, component) {
        let result;
        if (component) {
            result = this.scales.filter(s => s.component === component && s.output === output);
        }
        result = this.scales.filter(s => s.outputName === output);
        if (!result || result.length === 0) {
            messages_1.ReportDielUserError(`Scale not defined for ${output} ${component ? `for compoenetn ${component}` : ""}`, messages_1.UserErrorType.UndefinedScale);
        }
        else if (result.length > 1) {
            messages_1.ReportUserRuntimeWarning(`Output ${output} used for multiple ${component ? `for compoenetn ${component}` : ""}`);
        }
        const rawValue = result[0];
        if (dielUtils_1.CheckObjKeys(["dimension", "x", "y", "z"], rawValue)) {
            return rawValue;
        }
        else
            messages_1.LogInternalError(`scale logic inconsistent`);
        return null;
    }
    /**
     * Returns the name of derived view (which will be auto generated if not specified)
     * @param q: the raw query string
     * @param rName: the relation name
     */
    AddOutputRelationByString(q, rName) {
        return __awaiter(this, void 0, void 0, function* () {
            const relationSelection = compiler_2.ParsePlainSelectQueryAst(q);
            rName = rName ? rName : runtimeHelper_1.GenerateViewName(relationSelection);
            const derived = {
                rName,
                relationType: "Output" /* Output */,
                selection: relationSelection
            };
            yield this.AddViewByAst(derived);
            return rName;
        });
    }
    AddRelation(r) {
        const find = this.ast.relations.findIndex(oldR => r.rName === oldR.rName);
        if (DielAstGetters_1.IsRelationTypeDerived(r.relationType)) {
            if (find > -1) {
                if (r.replaces) {
                    this.DeleteView(find);
                }
                else {
                    // the distributed execution might attempt multiple times for the same thing
                    // just ignore for now
                    return messages_1.LogInternalWarning(`You are defining something already defined ${r.rName}`);
                }
            }
            this.AddViewByAst(r);
        }
        else {
            if (find > -1) {
                return messages_1.LogInternalWarning(`You are defining something already defined ${r.rName}`);
            }
            this.AddOriginalTableByAst(r);
        }
        return null;
    }
    DeleteView(foundIdx) {
        // take it off
        // and keep it if it were user created
        const r = this.ast.relations[foundIdx];
        if (r.origin === dielAstTypes_1.RelationOrigin.User) {
            // only save it if it were the original thing
            this.ast.replacedRelations.push(r);
        }
        this.ast.relations.splice(foundIdx, 1);
        // also need to remove them from the dependency tree
        this.physicalExecution.RemoveDerivedAst(r);
        // TODO: also from the executions in other DBs that have been shipped...
        return;
    }
    AddOriginalTableByAst(rDef) {
        return __awaiter(this, void 0, void 0, function* () {
            // we don't need to compile it...
            // note that the dependency tree will be fixed once we add derived views
            //   so not modified here.
            this.ast.relations.push(rDef);
        });
    }
    AddTemplateByString(q) {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    // this is accessed by the Notebook to reason with he ASTs
    // if this was an output view on an async event, then we need to access the underlying event...
    // so it has to be from the DIEL ast and not SQL ast
    GetRelationDef(rName) {
        return DielAstGetters_1.GetRelationDef(this.ast, rName);
    }
    /**
     * Note that the AST here need not be typed or de-stared
     * It's fine if they are.
     * #OPTIMIZE: In the future, we can skip some of the compiling steps if its already normalized
     */
    AddViewByAst(derived) {
        return __awaiter(this, void 0, void 0, function* () {
            const compiledAst = compiler_1.CompileDerivedAstGivenAst(this.ast, derived);
            // physical execution need to also maintain some metadata...
            this.physicalExecution.AddDerivedAst(compiledAst);
            const instructions = this.physicalExecution.GetInstructionsToAddOutput(compiledAst);
            if (instructions)
                yield this.incrementalExecuteToDb(instructions);
            // then set up the prepared statements as well (TODO: need to think thru all the steps that need to happen for the runtime, similar to the steps that compile DIEL)
            if (derived.relationType === "Output" /* Output */) {
                this.setupNewOutput(derived.rName);
            }
            // also should replace things in the dependency tree if it's specified
            if (derived.replaces) {
            }
        });
    }
    // ------------------------ debugging related ---------------------------
    inspectQueryResult(query) {
        let r = this.db.exec(query)[0];
        if (r) {
            console.log(r.columns.join("\t"));
            console.log(JSON.stringify(r.values).replace(/\],\[/g, "\n").replace("[[", "").replace("]]", "").replace(/,/g, "\t"));
            // console.table(r.columns);
            // console.table(r.values);
        }
        else {
            console.log("No results");
        }
        return r;
    }
    /**
     * Check if the view constraint query is broken. If so, report what records broke them.
     * @param query
     * @param viewName
     * @param constraint
     */
    reportConstraintQueryResult(query, viewName, constraint) {
        function reportBrokenConstraints(qr) {
            if (qr) {
                console.log(`%cConstraint Broken!\nview: ${viewName}\nconstraint: ${constraint}`, "background:red; color: white");
                console.log(qr.columns.join("\t"));
                console.log(JSON.stringify(qr.values).replace(/\],\[/g, "\n").replace("[[", "").replace("]]", "").replace(/,/g, "\t"));
            }
        }
        // 1. test if the views are already materialized!!
        // 2. if not materialized, evaluate constraints
        this.physicalExecution.sqlAstSpecPerDb.forEach((ast, id) => {
            ast.relations.forEach((relation) => {
                // 2-1. check which db the view is defined in.
                // correct db only if the relatin is view, event view, output
                if (relation.rName === viewName && relation.relationType === "View" /* View */) {
                    if (id === DielPhysicalExecution_1.LocalDbId) {
                        // local db; execute
                        try {
                            reportBrokenConstraints(this.db.exec(query)[0]);
                        }
                        catch (error) {
                            messages_1.LogInternalError(`Error while running view constraint query ${query}`);
                        }
                    }
                    else {
                        // remote db; ship the execute query
                        const remoteInstance = this.findRemoteDbEngine(id);
                        if (remoteInstance) {
                            const executeMessage = {
                                remoteAction: runtimeTypes_1.DielRemoteAction.GetResultsByPromise,
                                requestTimestep: this.timestep,
                                sql: query
                            };
                            remoteInstance.SendMsg(executeMessage, true).then((value) => {
                                if (value.results.length > 0) {
                                    reportBrokenConstraints(runtimeHelper_1.convertRelationObjectToQueryResults(value.results));
                                }
                            });
                        }
                        else {
                            messages_1.LogInternalError(`Remote ${id} is not found!`);
                        }
                    }
                }
            });
        });
    }
}
exports.default = DielRuntime;
//# sourceMappingURL=DielRuntime.js.map