"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dielAstTypes_1 = require("../parser/dielAstTypes");
const syntaxSugar_1 = require("../compiler/passes/syntaxSugar");
const messages_1 = require("../util/messages");
function getTimestepColumn(relationName, request = false) {
    return {
        exprType: dielAstTypes_1.ExprType.Column,
        dataType: dielAstTypes_1.DielDataType.Number,
        columnName: request ? dielAstTypes_1.BuiltInColumn.REQUEST_TIMESTEP : dielAstTypes_1.BuiltInColumn.TIMESTEP,
        relationName
    };
}
function defaultPolicyGetOutputHelper(ast, asyncViewName, eventDeps) {
    // const derivedColumnSelections = ast.selection.compositeSelections[0].relation.derivedColumnSelections;
    // const compositeSelections = ast.selection.compositeSelections.map((s) => {
    //   // LAZY
    //   const relation = JSON.parse(JSON.stringify(s.relation)) as SelectionUnit;
    //   // we just need to take the alias (assume normalized), and add the evenViewName
    const originalDerivedColumnSelections = ast.selection.compositeSelections[0].relation.derivedColumnSelections;
    if (!originalDerivedColumnSelections) {
        messages_1.LogInternalError(`AST for ${ast.rName} should be compiled`);
    }
    const derivedColumnSelections = originalDerivedColumnSelections.map(c => {
        if (!c.alias)
            messages_1.LogInternalError(`Alias should be normalized already!`);
        const newExpr = {
            exprType: dielAstTypes_1.ExprType.Column,
            columnName: c.alias,
            relationName: asyncViewName,
            dataType: c.expr.dataType,
        };
        return {
            expr: newExpr,
            alias: c.alias
        };
    });
    // if (eventDeps.size === 0) {
    const newOutput = {
        rName: ast.rName,
        relationType: "Output" /* Output */,
        selection: {
            astType: dielAstTypes_1.AstType.RelationSelection,
            compositeSelections: [{
                    op: dielAstTypes_1.SetOperator.NA,
                    relation: {
                        isDistinct: false,
                        derivedColumnSelections,
                        baseRelation: {
                            relationReferenceType: dielAstTypes_1.RelationReferenceType.Direct,
                            isLatest: true,
                            relationName: asyncViewName
                        }
                    }
                }]
        }
    };
    return syntaxSugar_1.ApplyLatestToDerivedRelation(newOutput);
    // FIXME: the following logic is a bit bogus...
    // } else {
    //   let timeStepSelections: SelectionUnit[] = [];
    //   eventDeps.forEach(e => {
    //     timeStepSelections.push({
    //       isDistinct: false,
    //       derivedColumnSelections: [{
    //         expr: getTimestepColumn(e),
    //         alias:  BuiltInColumn.TIMESTEP,
    //       }],
    //       baseRelation: {
    //         relationReferenceType: RelationReferenceType.Direct,
    //         relationName: e,
    //       }
    //     });
    //   });
    //   const unionedTimestepRelationName = "timestepUnion";
    //   const compositeSelections = timeStepSelections.map((relation, i) => {
    //     if (i) {
    //       return {
    //         op: SetOperator.UNION,
    //         relation
    //       };
    //     } else {
    //       return {
    //         op: SetOperator.NA,
    //         relation
    //       };
    //     }
    //   });
    //   const unionTimestepRelation: RelationSelection = {
    //     astType: AstType.RelationSelection,
    //     compositeSelections
    //   };
    //   const maxTimeStepRelationName = "maxTimeStep";
    //   const maxColumnSelection: RelationSelection = {
    //     astType: AstType.RelationSelection,
    //     compositeSelections: [{
    //       op: SetOperator.NA,
    //       relation: {
    //         derivedColumnSelections: [{
    //           expr: {
    //             exprType: ExprType.Column,
    //             dataType: DielDataType.Number,
    //             functionType: FunctionType.Math,
    //             functionReference: "max",
    //             args: [getTimestepColumn(unionedTimestepRelationName)]
    //           },
    //           alias: BuiltInColumn.TIMESTEP
    //         }],
    //         baseRelation: {
    //           relationReferenceType: RelationReferenceType.Subquery,
    //           subquery: unionTimestepRelation,
    //           alias: unionedTimestepRelationName
    //         }
    //       }
    //     }]
    //   };
    //   let joinClauses: JoinAst[] = [{
    //     astType: AstType.Join,
    //     joinType: JoinType.Inner,
    //     relation: {
    //       relationReferenceType: RelationReferenceType.Subquery,
    //       subquery: maxColumnSelection,
    //       alias: maxTimeStepRelationName
    //     },
    //     predicate: {
    //       exprType: ExprType.Func,
    //       dataType: DielDataType.Boolean,
    //       functionType: FunctionType.Compare,
    //       functionReference: "=",
    //       args: [getTimestepColumn(asyncViewName, true), getTimestepColumn(maxTimeStepRelationName)]
    //     }
    //   }];
    //   const newOutput: DerivedRelation = {
    //     rName: ast.rName,
    //     relationType: RelationType.Output,
    //     selection: {
    //       astType: AstType.RelationSelection,
    //       compositeSelections: [{
    //         op: SetOperator.NA,
    //         relation: {
    //           derivedColumnSelections,
    //           baseRelation: {
    //             relationReferenceType: RelationReferenceType.Direct,
    //             relationName: asyncViewName
    //           },
    //           joinClauses
    //         }
    //       }]
    //     }
    //   };
    //   return newOutput;
    // }
}
function GetAsyncViewNameFromOutput(rName) {
    return `${rName}AsyncView`;
}
exports.GetAsyncViewNameFromOutput = GetAsyncViewNameFromOutput;
/**
 * Note that we are passing the input dependencies here because
 * otherwise it needs to get access to the IR/depTree
 * an example would be
  SELECT *
  FROM distDataEvent e
    JOIN LATEST slideItx i ON i.timestep = e.request_timestep;

    where request_timestep = (select max timestep from slideItx union )
  we need to find the latest of all the inputs, if there are more than one
 * @param ast
 * @param inputDeps
 */
function OutputToAsyncDefaultPolicty(ast, eventDeps) {
    console.log("applying default async", ast.rName);
    // first create a new async view from the output
    // again, copy by reference, should be a bit careful here
    // note that we cannot _just_ copy over, the relation name would have changed.
    const viewName = GetAsyncViewNameFromOutput(ast.rName);
    const asyncView = {
        rName: viewName,
        relationType: "EventView" /* EventView */,
        selection: ast.selection
    };
    const output = defaultPolicyGetOutputHelper(ast, viewName, eventDeps);
    return {
        asyncView,
        output
    };
}
exports.OutputToAsyncDefaultPolicty = OutputToAsyncDefaultPolicty;
//# sourceMappingURL=asyncPolicies.js.map