"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dielAstTypes_1 = require("../parser/dielAstTypes");
const messages_1 = require("../util/messages");
function getNameFromExpr(e) {
    switch (e.exprType) {
        case dielAstTypes_1.ExprType.Column:
            return e.columnName;
        default:
            return "";
    }
}
function GenerateViewName(q) {
    const hash = Math.floor(Math.random() * 1000).toString();
    if (q.compositeSelections) {
        const hintRel = q.compositeSelections[0].relation.baseRelation
            ? "_" + q.compositeSelections[0].relation.baseRelation.alias
            : "";
        const hintCol = q.compositeSelections[0].relation.columnSelections
            .map(c => c.alias ? c.alias : getNameFromExpr(c.expr))
            .join("_");
        return `${hintCol}${hintRel}${hash}`;
    }
    return hash;
}
exports.GenerateViewName = GenerateViewName;
function CaughtLocalRun(db, s) {
    try {
        console.log(`%c Running Query in Main:\n${s}`, "color: purple");
        db.run(s);
    }
    catch (error) {
        messages_1.LogInternalError(`Error while running\n${s}\n${error}`);
    }
}
exports.CaughtLocalRun = CaughtLocalRun;
/**
 * returns all the SQL that defines tables in this DB
 *   so that we can add to IR parsing (for type inference and other processing needs)
 */
function getExistingTableDefinitions(isWorker, db) {
    let queries = "";
    const q = `SELECT name, sql FROM sqlite_master WHERE type='table'`;
    if (isWorker) {
        throw new Error(`not yet implemented`);
    }
    else {
        const done = () => { console.log("done"); };
        db.each(q, (o) => {
            queries += queries + o.sql;
        }, done);
    }
    return queries;
}
exports.getExistingTableDefinitions = getExistingTableDefinitions;
/**
 * Hack: some relations are defined already from another session---currently we don't have a hook to clean up, so skip them
 * , relationsToSkip?: string[]
 */
function processSqlMetaDataFromRelationObject(rO, sourceName) {
    // const filtered = (relationsToSkip)
    //   ? rO.filter(r => !relationsToSkip.find(rS => rS === r["name"]))
    //   : rO;
    return rO.map(definition => definition["sql"].toString()
        .replace(/create table/ig, `\n--${sourceName}\nregister table`) + ";")
        .join("\n");
}
exports.processSqlMetaDataFromRelationObject = processSqlMetaDataFromRelationObject;
function ParseSqlJsWorkerResult(data) {
    if (data && (data.length > 0) && data[0].values) {
        const o = data[0].values.map((v) => {
            let oi = {};
            v.map((vi, i) => {
                oi[data[0].columns[i]] = vi;
            });
            return oi;
        });
        return o;
    }
    return [];
}
exports.ParseSqlJsWorkerResult = ParseSqlJsWorkerResult;
/**
 * emulates better-sqlite's prepare('query').all()
 * @param db Database instance
 * @param query must be single line with no parameters
 */
function SqlJsGetObjectArrayFromQuery(db, query) {
    const stmt = db.prepare(query);
    stmt.bind({});
    let r = [];
    while (stmt.step()) {
        r.push(stmt.getAsObject());
    }
    return r;
}
exports.SqlJsGetObjectArrayFromQuery = SqlJsGetObjectArrayFromQuery;
// /**
//  * run time type checker
//  * @param fn
//  */
// export const checkType = fn => (params = []) => {
//   const { required } = fn;
//   const missing = required.filter(param => !(param in params));
//   if (missing.length) {
//     throw new Error(`${ fn.name }() Missing required parameter(s):
//     ${ missing.join(", ") }`);
//   }
//   return fn(params);
// };
function convertRelationObjectToQueryResults(ro) {
    let qr = {
        columns: [],
        values: []
    };
    qr.columns = Object.keys(ro[0]);
    ro.forEach((array) => {
        let values = [];
        qr.columns.forEach((colname) => {
            values.push(array[colname]);
        });
        qr.values.push(values);
    });
    return qr;
}
exports.convertRelationObjectToQueryResults = convertRelationObjectToQueryResults;
//# sourceMappingURL=runtimeHelper.js.map