"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sql_js_1 = require("sql.js");
// import initSqlJs from "sql.js";
const dielUdfs_1 = require("./dielUdfs");
const messages_1 = require("./messages");
function DeepCopy(o) {
    return JSON.parse(JSON.stringify(o));
}
exports.DeepCopy = DeepCopy;
function CheckObjKeys(required, obj) {
    for (let i = 0; i < required.length; i++) {
        if (!(required[i] in obj)) {
            return false;
        }
    }
    return true;
}
exports.CheckObjKeys = CheckObjKeys;
function SetSymmetricDifference(setA, setB) {
    let _difference = new Set(setA);
    for (let elem of setB) {
        if (_difference.has(elem)) {
            _difference.delete(elem);
        }
        else {
            _difference.add(elem);
        }
    }
    return _difference;
}
exports.SetSymmetricDifference = SetSymmetricDifference;
/**
 * find elements in A but not in B
 * @param setA
 * @param setB
 */
function SetDifference(setA, setB) {
    let _difference = new Set(setA);
    for (let elem of setB) {
        _difference.delete(elem);
    }
    return _difference;
}
exports.SetDifference = SetDifference;
function IsSuperset(superset, subset) {
    for (let elem of subset) {
        if (!superset.has(elem)) {
            return false;
        }
    }
    return true;
}
exports.IsSuperset = IsSuperset;
function IsSetIdentical(setA, setB) {
    return SetDifference(setA, setB).size === 0;
}
exports.IsSetIdentical = IsSetIdentical;
function SetUnion(setA, setB) {
    let _union = new Set(setA);
    for (let elem of setB) {
        _union.add(elem);
    }
    return _union;
}
exports.SetUnion = SetUnion;
function SetIntersection(setA, setB) {
    let _intersection = new Set();
    for (let elem of setB) {
        if (setA.has(elem)) {
            _intersection.add(elem);
        }
    }
    return _intersection;
}
exports.SetIntersection = SetIntersection;
function loadDbHelper(db, file, tick) {
    return __awaiter(this, void 0, void 0, function* () {
        if (db) {
            db.close();
        }
        let buffer;
        const response = yield fetch(file);
        const bufferRaw = yield response.arrayBuffer();
        buffer = new Uint8Array(bufferRaw);
        // const SQL = await initSqlJs();
        // db = new SQL.Database(buffer);
        db = new sql_js_1.Database(buffer);
        // db.create_function("timeNow", timeNow);
        db.create_function("log", dielUdfs_1.log);
        db.create_function("tick", tick());
        // db.create_function("tick", this.db);
        messages_1.LogInfo(`DIEL Loaded DB Successfully`);
    });
}
exports.loadDbHelper = loadDbHelper;
// console tools
// export function d(db: Database, sql: string) {
//   let r = db.exec(sql);
//   if (r.length > 0) {
//     r[0].values.map((v) => {
//       v.map((c, i) => {
//         if (r[0].columns[i] === "ts") {
//           c = new Date(c as number).toDateString();
//         }
//       });
//     });
//     console.log(r[0].columns.join("\t"));
//     console.log(JSON.stringify(r[0].values).replace(/\],\[/g, "\n").replace("[[", "").replace("]]", "").replace(/,/g, "\t"));
//   } else {
//     console.log("NO RESULT");
//   }
// }
// if (typeof window !== "undefined" && window) (<any>window).d = d;
// debug assertions
function assertQueryHasResult(r, query) {
    if ((!r) || (!r.values)) {
        throw new Error(`Query ${query} has NO result`);
    }
}
exports.assertQueryHasResult = assertQueryHasResult;
// this is needed because the groupby has non deterministic orderings
// and I wasn't able to write a custom reducer
// I suspect this is faster anyways
function hashCompare(a, b) {
    // split by -
    const aVals = a.split("-");
    const bVals = b.split("-");
    for (let i = 0; i < aVals.length; i++) {
        const av = aVals[i];
        const table = av.split(":")[0];
        const bR = bVals.filter(bv => bv.split(":")[0] === table);
        if (bR.length !== 1) {
            console.log("didn't find", table, bVals);
            return 0;
        }
        if (bR[0] !== av) {
            return 0;
        }
    }
    return 1;
}
exports.hashCompare = hashCompare;
/**********************
 * download support
 *********************/
function prettyTimeNow() {
    return new Date().toTimeString().substr(0, 8);
}
function downloadHelper(blob, name, extension = "db") {
    if (typeof window !== "undefined" && window) {
        let a = document.createElement("a");
        a.href = window.URL.createObjectURL(blob);
        a.download = `${name}_${prettyTimeNow()}.${extension}`;
        a.onclick = function () {
            setTimeout(function () {
                window.URL.revokeObjectURL(a.href);
            }, 1500);
        };
        a.click();
    }
    else {
        console.log("Not in web environment!");
    }
}
exports.downloadHelper = downloadHelper;
function downloadQueryResultAsCSV(db, query) {
    let csvContent = "";
    let r = db.exec(query);
    if (r.length && r[0].values) {
        csvContent += r[0].columns.join(",") + "\r\n";
        r[0].values.forEach((rowArray) => {
            let row = rowArray.join(",");
            csvContent += row + "\r\n";
        });
        let b = new Blob([csvContent], { type: "text/plain;charset=UTF-8" });
        downloadHelper(b, "userData.csv");
        console.log("should have downloaded", csvContent);
    }
    else {
        console.log("NO RESULT");
    }
}
exports.downloadQueryResultAsCSV = downloadQueryResultAsCSV;
//# sourceMappingURL=dielUtils.js.map