"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DielRuntime_1 = require("../runtime/DielRuntime");
exports.FgRed = "\x1b[31m";
exports.FgGreen = "\x1b[32m";
exports.FgBlue = "\x1b[34m";
exports.FgGray = "\x1b[90m";
exports.Underscore = "\x1b[4m";
exports.FgCyan = "\x1b[36m";
exports.FgMagenta = "\x1b[35m";
exports.BgRed = "\x1b[41m";
exports.BgGreen = "\x1b[42m";
exports.BgYellow = "\x1b[43m";
exports.Reset = "\x1b[0m";
var DielInternalErrorType;
(function (DielInternalErrorType) {
    DielInternalErrorType["Untitled"] = "Untitled";
    DielInternalErrorType["ArgNull"] = "ArgNull";
    DielInternalErrorType["TypeError"] = "TypeError";
    DielInternalErrorType["RelationNotFound"] = "RelationNotFound";
    DielInternalErrorType["NotImplemented"] = "NotImplemented";
    DielInternalErrorType["UnionTypeNotAllHandled"] = "UnionTypeNotAllHandled";
    DielInternalErrorType["MalFormedAst"] = "MalFormedAst";
    DielInternalErrorType["TestError"] = "TestError";
})(DielInternalErrorType = exports.DielInternalErrorType || (exports.DielInternalErrorType = {}));
var DielInternalWarningType;
(function (DielInternalWarningType) {
    DielInternalWarningType["Untitled"] = "Untitled";
    DielInternalWarningType["ArgNull"] = "ArgNull";
})(DielInternalWarningType = exports.DielInternalWarningType || (exports.DielInternalWarningType = {}));
function PrintCode(code) {
    const codeWithLine = code.split("\n").map((c, i) => `${i + 1}\t${c}`).join("\n");
    console.log(`DIEL Code\n%s}`, codeWithLine);
}
exports.PrintCode = PrintCode;
function LogSetup(m) {
    console.log(`${exports.FgMagenta}${m}${exports.Reset}`);
}
exports.LogSetup = LogSetup;
function LogExecutionTrace(m, obj) {
    // return null;
    console.log(`${exports.FgGray}%s${exports.Reset}`, m, obj ? obj : "");
}
exports.LogExecutionTrace = LogExecutionTrace;
function LogTest(m, obj) {
    console.log(`${exports.FgCyan}%s${exports.Reset}`, m, obj ? obj : "");
}
exports.LogTest = LogTest;
function LogInternalError(m, errorType = DielInternalErrorType.Untitled) {
    const message = `Error[${errorType}]: ${m}`;
    if (DielRuntime_1.STRICT) {
        debugger;
        throw new Error(message);
    }
    else {
        console.log(`${exports.FgRed}${message}${exports.Reset}`);
    }
    return null;
}
exports.LogInternalError = LogInternalError;
function LogInternalWarning(m, wariningType = DielInternalWarningType.Untitled) {
    console.log(`${exports.FgMagenta}[${wariningType}]%s${exports.Reset}`, m);
}
exports.LogInternalWarning = LogInternalWarning;
function LogInfo(m, obj) {
    if (obj) {
        console.log(`${exports.FgBlue}%s${exports.Reset}`, m, obj);
    }
    else {
        console.log(`${exports.FgBlue}%s${exports.Reset}`, m);
    }
}
exports.LogInfo = LogInfo;
function LogTmp(m) {
    console.log(`${exports.FgGray}${m}${exports.Reset}`);
}
exports.LogTmp = LogTmp;
function LogStandout(m) {
    console.log(`${exports.BgYellow}%s${exports.Reset}`, m);
}
exports.LogStandout = LogStandout;
function ReportDielBasicParsingError(m) {
    console.log(`${exports.FgRed}Parsing error from user provided code:\n%s${exports.Reset}`, m);
    if (DielRuntime_1.STRICT)
        throw new Error();
}
exports.ReportDielBasicParsingError = ReportDielBasicParsingError;
function ReportUserRuntimeWarning(m) {
    console.log(`${exports.FgMagenta}[User Error]%s${exports.Reset}`, m);
    return null;
}
exports.ReportUserRuntimeWarning = ReportUserRuntimeWarning;
// note that this is in browser
function ReportUserRuntimeError(m) {
    if (DielRuntime_1.STRICT)
        throw new Error(m);
    console.log(`${exports.BgYellow}[User Error]%s${exports.Reset}`, m);
    return null;
}
exports.ReportUserRuntimeError = ReportUserRuntimeError;
// both warning and error
var UserErrorType;
(function (UserErrorType) {
    UserErrorType["UndefinedScale"] = "UndefinedScale";
})(UserErrorType = exports.UserErrorType || (exports.UserErrorType = {}));
// TODO: this should also report the line of the code
// the input should be more structured
function ReportDielUserError(m, q, errorType) {
    console.log(`${exports.FgRed}%s${exports.Reset}`, m);
    if (DielRuntime_1.STRICT)
        throw new Error();
    return null;
}
exports.ReportDielUserError = ReportDielUserError;
function ReportDielUserWarning(m, q) {
    console.log(`User Warning: ${exports.FgMagenta}%s${exports.Reset}`, m);
    if (q)
        console.log(`\nQuery: ${exports.FgMagenta}%s${exports.Reset}\n`, q);
}
exports.ReportDielUserWarning = ReportDielUserWarning;
function sanityAssert(b, msg) {
    if (!b) {
        LogInternalError(`Assertion error: ${msg}`);
    }
}
exports.sanityAssert = sanityAssert;
//# sourceMappingURL=messages.js.map